@@ -200,7 +200,9 @@ attr_line_t::insert(size_t index,
         this->al_string, starting_line_index, this->al_string.length());
     string_fragment last_word;
     ssize_t line_ch_count = 0;
+    ssize_t line_indent_count = 0;
     auto needs_indent = false;
+    auto last_was_pre = false;
 
     while (!text_to_wrap.empty()) {
         if (needs_indent) {
@@ -214,6 +216,7 @@ attr_line_t::insert(size_t index,
             split_attrs(*this, indent_lr);
             indent_lr.lr_end += tws->tws_padding_indent;
             line_ch_count += tws->tws_padding_indent;
+            line_indent_count += tws->tws_padding_indent;
             if (!indent_lr.empty()) {
                 this->al_attrs.emplace_back(indent_lr, SA_PREFORMATTED.value());
             }
@@ -222,18 +225,34 @@ attr_line_t::insert(size_t index,
                 tws->tws_indent + tws->tws_padding_indent);
             needs_indent = false;
         }
-        auto chunk = text_stream::consume(text_to_wrap);
 
-        text_to_wrap = chunk.match(
+        text_stream::chunk next_chunk(mapbox::util::no_init{});
+        auto pre_iter = find_string_attr_containing(
+            this->al_attrs, &SA_PREFORMATTED, text_to_wrap.sf_begin);
+        if (pre_iter != this->al_attrs.end()) {
+            auto pre_len = pre_iter->sa_range.lr_end - text_to_wrap.sf_begin;
+            auto pre_lf = text_to_wrap.find('\n');
+            if (pre_lf && pre_lf.value() < pre_len) {
+                pre_len = pre_lf.value() + 1;
+            }
+
+            auto pre_pair = text_to_wrap.split_n(pre_len);
+            next_chunk = text_stream::word{
+                pre_pair->first,
+                pre_pair->second,
+            };
+        }
+        if (!next_chunk.valid()) {
+            next_chunk = text_stream::consume(text_to_wrap);
+        }
+
+        text_to_wrap = next_chunk.match(
             [&](text_stream::word word) {
                 auto ch_count
                     = word.w_word.utf8_length().unwrapOr(word.w_word.length());
 
-                if ((line_ch_count + ch_count) > usable_width
-                    && find_string_attr_containing(this->al_attrs,
-                                                   &SA_PREFORMATTED,
-                                                   text_to_wrap.sf_begin)
-                        == this->al_attrs.end())
+                if (line_ch_count > line_indent_count && !last_was_pre
+                    && (line_ch_count + ch_count) > usable_width)
                 {
                     this->insert(word.w_word.sf_begin, 1, '\n');
                     this->insert(word.w_word.sf_begin + 1,
@@ -250,6 +269,7 @@ attr_line_t::insert(size_t index,
                                                     SA_PREFORMATTED.value());
                     }
                     line_ch_count = tws->tws_padding_indent + ch_count;
+                    line_indent_count = tws->tws_padding_indent;
                     auto trailing_space_count = 0;
                     if (!last_word.empty()) {
                         trailing_space_count
@@ -263,12 +283,18 @@ attr_line_t::insert(size_t index,
                                  1 + tws->tws_indent + tws->tws_padding_indent);
                 }
                 line_ch_count += ch_count;
+                if (word.w_word.endswith("\n")) {
+                    line_ch_count = 0;
+                    line_indent_count = 0;
+                    needs_indent = true;
+                }
 
                 return word.w_remaining;
             },
             [&](text_stream::space space) {
                 if (space.s_value == "\n") {
                     line_ch_count = 0;
+                    line_indent_count = 0;
                     needs_indent = true;
                     return space.s_remaining;
                 }
@@ -287,6 +313,7 @@ attr_line_t::insert(size_t index,
                                     space.s_value.length());
                         this->insert(space.s_value.sf_begin, "\n");
                         line_ch_count = 0;
+                        line_indent_count = 0;
                         needs_indent = true;
 
                         auto trailing_space_count = 0;
@@ -318,9 +345,10 @@ attr_line_t::insert(size_t index,
             [](text_stream::corrupt corrupt) { return corrupt.c_remaining; },
             [](text_stream::eof eof) { return eof.e_remaining; });
 
-        if (chunk.is<text_stream::word>()) {
+        if (next_chunk.is<text_stream::word>()) {
             last_word = text_to_wrap;
         }
+        last_was_pre = (pre_iter != this->al_attrs.end());
 
         ensure(this->al_string.data() == text_to_wrap.sf_string);
         ensure(text_to_wrap.sf_begin <= text_to_wrap.sf_end);
@@ -328,6 +356,19 @@ attr_line_t::insert(size_t index,
     return *this;
 }
 
+attr_line_t&
+attr_line_t::wrap_with(text_wrap_settings* tws)
+{
+    attr_line_t tmp;
+
+    tmp.al_string = std::move(this->al_string);
+    tmp.al_attrs = std::move(this->al_attrs);
+
+    this->append(tmp, tws);
+
+    return *this;
+}
+
 attr_line_t
 attr_line_t::subline(size_t start, size_t len) const
 {
@@ -421,15 +462,15 @@ attr_line_t::apply_hide()
 
             this->al_string.replace(lr.lr_start, lr.length(), "\xE2\x8B\xAE");
             shift_string_attrs(sa, lr.lr_start + 1, -(lr.length() - 3));
-            sattr.sa_type = &VC_ROLE;
-            sattr.sa_value = role_t::VCR_HIDDEN;
+            sattr.sa_type = &VC_ICON;
+            sattr.sa_value = ui_icon_t::hidden;
             lr.lr_end = lr.lr_start + 3;
         }
     }
 }
 
 attr_line_t&
-attr_line_t::rtrim()
+attr_line_t::rtrim(std::optional<const char*> chars)
 {
     auto index = this->al_string.length();
 
@@ -440,7 +481,12 @@ attr_line_t::rtrim()
         {
             break;
         }
-        if (!isspace(this->al_string[index - 1])) {
+        if (chars
+            && strchr(chars.value(), this->al_string[index - 1]) == nullptr)
+        {
+            break;
+        }
+        if (!chars && !isspace(this->al_string[index - 1])) {
             break;
         }
     }
@@ -462,7 +508,9 @@ attr_line_t::erase(size_t pos, size_t len)
 
     this->al_string.erase(pos, len);
 
-    shift_string_attrs(this->al_attrs, pos, -((int32_t) len));
+    shift_string_attrs(this->al_attrs,
+                       line_range{(int) pos, (int) (pos + len)},
+                       -((int32_t) len));
     auto new_end = std::remove_if(
         this->al_attrs.begin(), this->al_attrs.end(), [](const auto& attr) {
             return attr.sa_range.empty();
@@ -475,7 +523,7 @@ attr_line_t::erase(size_t pos, size_t len)
 attr_line_t&
 attr_line_t::pad_to(ssize_t size)
 {
-    const auto curr_len = this->utf8_length_or_length();
+    const auto curr_len = this->column_width();
 
     if (curr_len < size) {
         this->append((size - curr_len), ' ');
@@ -505,6 +553,33 @@ line_range::intersection(const line_range& other) const
     return line_range{std::max(this->lr_start, other.lr_start), actual_end};
 }
 
+line_range&
+line_range::shift_range(const line_range& cover, int32_t amount)
+{
+    if (cover.lr_end <= this->lr_start) {
+        this->lr_start = std::max(0, this->lr_start + amount);
+        if (this->lr_end != -1) {
+            this->lr_end = std::max(0, this->lr_end + amount);
+        }
+    } else {
+        if (amount < 0 && cover.contains(*this)) {
+            this->lr_start = cover.lr_start;
+        }
+        if (this->lr_end != -1) {
+            if (cover.lr_start < this->lr_end) {
+                if (amount < 0 && amount < (cover.lr_start - this->lr_end)) {
+                    this->lr_end = cover.lr_start;
+                } else {
+                    this->lr_end
+                        = std::max(this->lr_start, this->lr_end + amount);
+                }
+            }
+        }
+    }
+
+    return *this;
+}
+
 line_range&
 line_range::shift(int32_t start, int32_t amount)
 {
@@ -535,3 +610,119 @@ line_range::shift(int32_t start, int32_t amount)
 
     return *this;
 }
+
+string_attrs_t::const_iterator
+find_string_attr(const string_attrs_t& sa, size_t near)
+{
+    auto nearest = sa.end();
+    ssize_t last_diff = INT_MAX;
+
+    for (auto iter = sa.begin(); iter != sa.end(); ++iter) {
+        const auto& lr = iter->sa_range;
+
+        if (!lr.is_valid() || !lr.contains(near)) {
+            continue;
+        }
+
+        ssize_t diff = near - lr.lr_start;
+        if (diff < last_diff) {
+            last_diff = diff;
+            nearest = iter;
+        }
+    }
+
+    return nearest;
+}
+
+void
+shift_string_attrs(string_attrs_t& sa, int32_t start, int32_t amount)
+{
+    for (auto& iter : sa) {
+        iter.sa_range.shift(start, amount);
+    }
+}
+
+void
+shift_string_attrs(string_attrs_t& sa, const line_range& cover, int32_t amount)
+{
+    for (auto& iter : sa) {
+        iter.sa_range.shift_range(cover, amount);
+    }
+}
+
+struct line_range
+find_string_attr_range(const string_attrs_t& sa, string_attr_type_base* type)
+{
+    auto iter = find_string_attr(sa, type);
+
+    if (iter != sa.end()) {
+        return iter->sa_range;
+    }
+
+    return line_range();
+}
+
+void
+remove_string_attr(string_attrs_t& sa, const line_range& lr)
+{
+    string_attrs_t::iterator iter;
+
+    while ((iter = find_string_attr(sa, lr)) != sa.end()) {
+        sa.erase(iter);
+    }
+}
+
+void
+remove_string_attr(string_attrs_t& sa, string_attr_type_base* type)
+{
+    for (auto iter = sa.begin(); iter != sa.end();) {
+        if (iter->sa_type == type) {
+            iter = sa.erase(iter);
+        } else {
+            ++iter;
+        }
+    }
+}
+
+string_attrs_t::iterator
+find_string_attr(string_attrs_t& sa, const line_range& lr)
+{
+    string_attrs_t::iterator iter;
+
+    for (iter = sa.begin(); iter != sa.end(); ++iter) {
+        if (lr.contains(iter->sa_range)) {
+            break;
+        }
+    }
+
+    return iter;
+}
+
+string_attrs_t::const_iterator
+find_string_attr(const string_attrs_t& sa,
+                 const string_attr_type_base* type,
+                 int start)
+{
+    auto iter = sa.begin();
+    for (; iter != sa.end(); ++iter) {
+        if (iter->sa_type == type && iter->sa_range.lr_start >= start) {
+            break;
+        }
+    }
+
+    return iter;
+}
+
+std::optional<const string_attr*>
+get_string_attr(const string_attrs_t& sa,
+                const string_attr_type_base* type,
+                int start)
+{
+    auto iter = find_string_attr(sa, type, start);
+
+    if (iter == sa.end()) {
+        return std::nullopt;
+    }
+
+    return std::make_optional(&(*iter));
+}
