@@ -45,6 +45,45 @@ using namespace lnav::roles::literals;
 namespace lnav {
 namespace console {
 
+snippet
+snippet::from_content_with_offset(intern_string_t src,
+                                  const attr_line_t& content,
+                                  size_t offset,
+                                  const std::string& errmsg)
+{
+    const auto content_sf = string_fragment::from_str(content.get_string());
+    const auto line_with_error = content_sf.find_boundaries_around(
+        offset, string_fragment::tag1{'\n'});
+    const auto line_with_context = content_sf.find_boundaries_around(
+        offset, string_fragment::tag1{'\n'}, 3);
+    const auto line_number = content_sf.sub_range(0, offset).count('\n');
+    const auto erroff_in_line = offset - line_with_error.sf_begin;
+
+    attr_line_t pointer;
+
+    pointer.append(erroff_in_line, ' ')
+        .append("^ "_snippet_border)
+        .append(lnav::roles::error(errmsg))
+        .with_attr_for_all(VC_ROLE.value(role_t::VCR_QUOTED_CODE));
+
+    snippet retval;
+    retval.s_content
+        = content.subline(line_with_context.sf_begin,
+                          line_with_error.sf_end - line_with_context.sf_begin);
+    if (line_with_error.sf_end >= retval.s_content.get_string().size()) {
+        retval.s_content.append("\n");
+    }
+    retval.s_content.append(pointer).append(
+        content.subline(line_with_error.sf_end,
+                        line_with_context.sf_end - line_with_error.sf_end));
+    retval.s_location = source_location{
+        src,
+        static_cast<int32_t>(1 + line_number),
+    };
+
+    return retval;
+}
+
 user_message
 user_message::raw(const attr_line_t& al)
 {
@@ -213,7 +252,7 @@ user_message::to_attr_line(std::set<render_flags> flags) const
     return retval;
 }
 
-static nonstd::optional<fmt::terminal_color>
+static std::optional<fmt::terminal_color>
 curses_color_to_terminal_color(int curses_color)
 {
     switch (curses_color) {
@@ -234,23 +273,145 @@ curses_color_to_terminal_color(int curses_color)
         case COLOR_RED:
             return fmt::terminal_color::red;
         default:
-            return nonstd::nullopt;
+            return std::nullopt;
+    }
+}
+
+static bool
+get_no_color()
+{
+    return getenv("NO_COLOR") != nullptr;
+}
+
+static bool
+get_yes_color()
+{
+    return getenv("YES_COLOR") != nullptr;
+}
+
+static bool
+get_fd_tty(int fd)
+{
+    return isatty(fd);
+}
+
+static void
+role_to_style(const role_t role,
+              fmt::text_style& default_bg_style,
+              fmt::text_style& default_fg_style,
+              fmt::text_style& line_style)
+{
+    switch (role) {
+        case role_t::VCR_TEXT:
+        case role_t::VCR_IDENTIFIER:
+            break;
+        case role_t::VCR_ALT_ROW:
+            line_style |= fmt::emphasis::bold;
+            break;
+        case role_t::VCR_SEARCH:
+            line_style |= fmt::emphasis::reverse;
+            break;
+        case role_t::VCR_ERROR:
+        case role_t::VCR_DIFF_DELETE:
+            line_style
+                |= fmt::fg(fmt::terminal_color::red) | fmt::emphasis::bold;
+            break;
+        case role_t::VCR_WARNING:
+        case role_t::VCR_RE_REPEAT:
+            line_style |= fmt::fg(fmt::terminal_color::yellow);
+            break;
+        case role_t::VCR_COMMENT:
+        case role_t::VCR_DIFF_ADD:
+            line_style |= fmt::fg(fmt::terminal_color::green);
+            break;
+        case role_t::VCR_SNIPPET_BORDER:
+            line_style |= fmt::fg(fmt::terminal_color::cyan);
+            break;
+        case role_t::VCR_OK:
+            line_style
+                |= fmt::emphasis::bold | fmt::fg(fmt::terminal_color::green);
+            break;
+        case role_t::VCR_FOOTNOTE_BORDER:
+            line_style |= fmt::fg(fmt::terminal_color::blue);
+            break;
+        case role_t::VCR_INFO:
+        case role_t::VCR_STATUS:
+            line_style
+                |= fmt::emphasis::bold | fmt::fg(fmt::terminal_color::magenta);
+            break;
+        case role_t::VCR_KEYWORD:
+        case role_t::VCR_RE_SPECIAL:
+            line_style
+                |= fmt::emphasis::bold | fmt::fg(fmt::terminal_color::cyan);
+            break;
+        case role_t::VCR_STRING:
+            line_style |= fmt::fg(fmt::terminal_color::magenta);
+            break;
+        case role_t::VCR_VARIABLE:
+            line_style |= fmt::emphasis::underline;
+            break;
+        case role_t::VCR_SYMBOL:
+        case role_t::VCR_NUMBER:
+        case role_t::VCR_FILE:
+            line_style |= fmt::emphasis::bold;
+            break;
+        case role_t::VCR_H1:
+            line_style
+                |= fmt::emphasis::bold | fmt::fg(fmt::terminal_color::magenta);
+            break;
+        case role_t::VCR_H2:
+            line_style |= fmt::emphasis::bold;
+            break;
+        case role_t::VCR_H3:
+        case role_t::VCR_H4:
+        case role_t::VCR_H5:
+        case role_t::VCR_H6:
+            line_style |= fmt::emphasis::underline;
+            break;
+        case role_t::VCR_LIST_GLYPH:
+            line_style |= fmt::fg(fmt::terminal_color::yellow);
+            break;
+        case role_t::VCR_INLINE_CODE:
+        case role_t::VCR_QUOTED_CODE:
+            default_fg_style = fmt::fg(fmt::terminal_color::white);
+            default_bg_style = fmt::bg(fmt::terminal_color::black);
+            break;
+        case role_t::VCR_LOW_THRESHOLD:
+            line_style |= fmt::bg(fmt::terminal_color::green);
+            break;
+        case role_t::VCR_MED_THRESHOLD:
+            line_style |= fmt::bg(fmt::terminal_color::yellow);
+            break;
+        case role_t::VCR_HIGH_THRESHOLD:
+            line_style |= fmt::bg(fmt::terminal_color::red);
+            break;
+        default:
+            // log_debug("missing role handler %d", (int) role);
+            break;
     }
 }
 
 void
 println(FILE* file, const attr_line_t& al)
 {
+    static const auto IS_NO_COLOR = get_no_color();
+    static const auto IS_YES_COLOR = get_yes_color();
+    static const auto IS_STDOUT_TTY = get_fd_tty(STDOUT_FILENO);
+    static const auto IS_STDERR_TTY = get_fd_tty(STDERR_FILENO);
+    static auto& vc = view_colors::singleton();
+
     const auto& str = al.get_string();
 
-    if (getenv("NO_COLOR") != nullptr
-        || (!isatty(fileno(file)) && getenv("YES_COLOR") == nullptr))
+    if (IS_NO_COLOR || (file != stdout && file != stderr)
+        || (((file == stdout && !IS_STDOUT_TTY)
+             || (file == stderr && !IS_STDERR_TTY))
+            && !IS_YES_COLOR))
     {
         fmt::print(file, "{}\n", str);
         return;
     }
 
-    std::set<size_t> points = {0, static_cast<size_t>(al.length())};
+    auto points = std::set<size_t>{0, static_cast<size_t>(al.length())};
 
     for (const auto& attr : al.get_attrs()) {
         if (!attr.sa_range.is_valid()) {
@@ -262,7 +423,7 @@ println(FILE* file, const attr_line_t& al)
         }
     }
 
-    nonstd::optional<size_t> last_point;
+    std::optional<size_t> last_point;
     for (const auto& point : points) {
         if (!last_point) {
             last_point = point;
@@ -273,6 +434,8 @@ println(FILE* file, const attr_line_t& al)
         auto line_style = fmt::text_style{};
         auto fg_style = fmt::text_style{};
         auto start = last_point.value();
+        std::optional<std::string> href;
+        auto replaced = false;
 
         for (const auto& attr : al.get_attrs()) {
             if (!attr.sa_range.contains(start)
@@ -282,7 +445,25 @@ println(FILE* file, const attr_line_t& al)
             }
 
             try {
-                if (attr.sa_type == &VC_BACKGROUND) {
+                if (attr.sa_type == &VC_ICON) {
+                    auto ic = attr.sa_value.get<ui_icon_t>();
+                    auto be = vc.wchar_for_icon(ic);
+                    auto icon_fg_style = default_fg_style;
+                    auto icon_bg_style = default_bg_style;
+                    auto icon_style = line_style;
+                    std::string utf8_out;
+
+                    role_to_style(
+                        be.role, icon_bg_style, icon_fg_style, icon_style);
+                    ww898::utf::utf8::write(
+                        be.value,
+                        [&utf8_out](const char ch) { utf8_out.push_back(ch); });
+                    fmt::print(file, icon_style, FMT_STRING("{}"), utf8_out);
+                    replaced = true;
+                } else if (attr.sa_type == &VC_HYPERLINK) {
+                    auto saw = string_attr_wrapper<std::string>(&attr);
+                    href = saw.get();
+                } else if (attr.sa_type == &VC_BACKGROUND) {
                     auto saw = string_attr_wrapper<int64_t>(&attr);
                     auto color_opt = curses_color_to_terminal_color(saw.get());
 
@@ -341,91 +522,14 @@ println(FILE* file, const attr_line_t& al)
                         default:
                             break;
                     }
-                } else if (attr.sa_type == &VC_ROLE) {
+                } else if (attr.sa_type == &VC_ROLE
+                           || attr.sa_type == &VC_ROLE_FG)
+                {
                     auto saw = string_attr_wrapper<role_t>(&attr);
                     auto role = saw.get();
 
-                    switch (role) {
-                        case role_t::VCR_TEXT:
-                        case role_t::VCR_IDENTIFIER:
-                            break;
-                        case role_t::VCR_SEARCH:
-                            line_style |= fmt::emphasis::reverse;
-                            break;
-                        case role_t::VCR_ERROR:
-                            line_style |= fmt::fg(fmt::terminal_color::red)
-                                | fmt::emphasis::bold;
-                            break;
-                        case role_t::VCR_WARNING:
-                        case role_t::VCR_RE_REPEAT:
-                            line_style |= fmt::fg(fmt::terminal_color::yellow);
-                            break;
-                        case role_t::VCR_COMMENT:
-                            line_style |= fmt::fg(fmt::terminal_color::green);
-                            break;
-                        case role_t::VCR_SNIPPET_BORDER:
-                            line_style |= fmt::fg(fmt::terminal_color::cyan);
-                            break;
-                        case role_t::VCR_OK:
-                            line_style |= fmt::emphasis::bold
-                                | fmt::fg(fmt::terminal_color::green);
-                            break;
-                        case role_t::VCR_INFO:
-                        case role_t::VCR_STATUS:
-                            line_style |= fmt::emphasis::bold
-                                | fmt::fg(fmt::terminal_color::magenta);
-                            break;
-                        case role_t::VCR_KEYWORD:
-                        case role_t::VCR_RE_SPECIAL:
-                            line_style |= fmt::emphasis::bold
-                                | fmt::fg(fmt::terminal_color::cyan);
-                            break;
-                        case role_t::VCR_STRING:
-                            line_style |= fmt::fg(fmt::terminal_color::magenta);
-                            break;
-                        case role_t::VCR_VARIABLE:
-                            line_style |= fmt::emphasis::underline;
-                            break;
-                        case role_t::VCR_SYMBOL:
-                        case role_t::VCR_NUMBER:
-                        case role_t::VCR_FILE:
-                            line_style |= fmt::emphasis::bold;
-                            break;
-                        case role_t::VCR_H1:
-                            line_style |= fmt::emphasis::bold
-                                | fmt::fg(fmt::terminal_color::magenta);
-                            break;
-                        case role_t::VCR_H2:
-                            line_style |= fmt::emphasis::bold;
-                            break;
-                        case role_t::VCR_H3:
-                        case role_t::VCR_H4:
-                        case role_t::VCR_H5:
-                        case role_t::VCR_H6:
-                            line_style |= fmt::emphasis::underline;
-                            break;
-                        case role_t::VCR_LIST_GLYPH:
-                            line_style |= fmt::fg(fmt::terminal_color::yellow);
-                            break;
-                        case role_t::VCR_QUOTED_CODE:
-                            default_fg_style
-                                = fmt::fg(fmt::terminal_color::white);
-                            default_bg_style
-                                = fmt::bg(fmt::terminal_color::black);
-                            break;
-                        case role_t::VCR_LOW_THRESHOLD:
-                            line_style |= fmt::bg(fmt::terminal_color::green);
-                            break;
-                        case role_t::VCR_MED_THRESHOLD:
-                            line_style |= fmt::bg(fmt::terminal_color::yellow);
-                            break;
-                        case role_t::VCR_HIGH_THRESHOLD:
-                            line_style |= fmt::bg(fmt::terminal_color::red);
-                            break;
-                        default:
-                            // log_debug("missing role handler %d", (int) role);
-                            break;
-                    }
+                    role_to_style(
+                        role, default_bg_style, default_fg_style, line_style);
                 }
             } catch (const fmt::format_error& e) {
                 log_error("style error: %s", e.what());
@@ -442,12 +546,69 @@ println(FILE* file, const attr_line_t& al)
             line_style |= default_bg_style;
         }
 
-        if (start < str.size()) {
+        if (line_style.has_foreground() && line_style.has_background()
+            && !line_style.get_foreground().is_rgb
+            && !line_style.get_background().is_rgb
+            && line_style.get_foreground().value.term_color
+                == line_style.get_background().value.term_color)
+        {
+            auto new_style = fmt::text_style{};
+
+            if (line_style.has_emphasis()) {
+                new_style |= line_style.get_emphasis();
+            }
+            new_style |= fmt::fg(line_style.get_foreground());
+            if (line_style.get_background().value.term_color
+                == lnav::enums::to_underlying(fmt::terminal_color::black))
+            {
+                new_style |= fmt::bg(fmt::terminal_color::white);
+            } else {
+                new_style |= fmt::bg(fmt::terminal_color::black);
+            }
+            line_style = new_style;
+        }
+
+        if (href) {
+            fmt::print(file, FMT_STRING("\x1b]8;;{}\x1b\\"), href.value());
+        }
+        if (!replaced && start < str.size()) {
             auto actual_end = std::min(str.size(), static_cast<size_t>(point));
-            fmt::print(file,
-                       line_style,
-                       FMT_STRING("{}"),
-                       str.substr(start, actual_end - start));
+            auto sub = std::string{};
+
+            for (auto lpc = start; lpc < actual_end;) {
+                auto cp_start = lpc;
+                auto read_res = ww898::utf::utf8::read(
+                    [&str, &lpc]() { return str[lpc++]; });
+
+                if (read_res.isErr()) {
+                    sub.append(fmt::format(
+                        FMT_STRING("{:?}"),
+                        fmt::string_view{&str[cp_start], lpc - cp_start}));
+                    continue;
+                }
+
+                auto ch = read_res.unwrap();
+                switch (ch) {
+                    case '\b':
+                        sub.append("\u232b");
+                        break;
+                    case '\x1b':
+                        sub.append("\u238b");
+                        break;
+                    case '\x07':
+                        sub.append("\U0001F514");
+                        break;
+
+                    default:
+                        sub.append(&str[cp_start], lpc - cp_start);
+                        break;
+                }
+            }
+
+            fmt::print(file, line_style, FMT_STRING("{}"), sub);
+        }
+        if (href) {
+            fmt::print(file, FMT_STRING("\x1b]8;;\x1b\\"));
         }
         last_point = point;
     }
