@@ -32,12 +32,15 @@
 #include "command_executor.hh"
 
 #include "base/ansi_scrubber.hh"
+#include "base/ansi_vars.hh"
 #include "base/fs_util.hh"
 #include "base/injector.hh"
 #include "base/itertools.hh"
+#include "base/paths.hh"
 #include "base/string_util.hh"
 #include "bound_tags.hh"
 #include "config.h"
+#include "curl_looper.hh"
 #include "db_sub_source.hh"
 #include "help_text_formatter.hh"
 #include "lnav.hh"
@@ -45,24 +48,23 @@
 #include "lnav_config.hh"
 #include "lnav_util.hh"
 #include "log_format_loader.hh"
+#include "prql-modules.h"
 #include "readline_highlighters.hh"
 #include "service_tags.hh"
 #include "shlex.hh"
+#include "sql_help.hh"
 #include "sql_util.hh"
 #include "vtab_module.hh"
-#include "yajlpp/json_ptr.hh"
 
+#ifdef HAVE_RUST_DEPS
+#    include "prqlc.cxx.hh"
+#endif
+
+using namespace std::literals::chrono_literals;
 using namespace lnav::roles::literals;
 
 exec_context INIT_EXEC_CONTEXT;
 
-static const std::string MSG_FORMAT_STMT = R"(
-SELECT count(*) AS total, min(log_line) AS log_line, log_msg_format
-    FROM all_logs
-    GROUP BY log_msg_format
-    ORDER BY total DESC
-)";
-
 int
 sql_progress(const struct log_cursor& lc)
 {
@@ -105,7 +107,7 @@ sql_progress_finished()
 
 static Result<std::string, lnav::console::user_message> execute_from_file(
     exec_context& ec,
-    const ghc::filesystem::path& path,
+    const std::string& src,
     int line_number,
     const std::string& cmdline);
 
@@ -160,20 +162,14 @@ bind_sql_parameters(exec_context& ec, sqlite3_stmt* stmt)
                           "are supported")
                       .with_help(
                           "named parameters start with a dollar-sign "
-                          "($) or colon (:) followed by the variable name");
+                          "($) or colon (:) followed by the variable name")
+                      .move();
             ec.add_error_context(um);
 
             return Err(um);
         }
 
-        ov_iter = ec.ec_override.find(name);
-        if (ov_iter != ec.ec_override.end()) {
-            sqlite3_bind_text(stmt,
-                              lpc,
-                              ov_iter->second.c_str(),
-                              ov_iter->second.length(),
-                              SQLITE_TRANSIENT);
-        } else if (name[0] == '$') {
+        if (name[0] == '$') {
             const auto& lvars = ec.ec_local_vars.top();
             const auto& gvars = ec.ec_global_vars;
             std::map<std::string, scoped_value_t>::const_iterator local_var,
@@ -251,26 +247,70 @@ bind_sql_parameters(exec_context& ec, sqlite3_stmt* stmt)
 static void
 execute_search(const std::string& search_cmd)
 {
-    lnav_data.ld_view_stack.top() | [&search_cmd](auto tc) {
-        auto search_term
-            = string_fragment(search_cmd)
-                  .find_right_boundary(0, string_fragment::tag1{'\n'})
-                  .to_string();
-        tc->execute_search(search_term);
-    };
+    textview_curses* tc = get_textview_for_mode(lnav_data.ld_mode);
+    auto search_term = string_fragment(search_cmd)
+                           .find_right_boundary(0, string_fragment::tag1{'\n'})
+                           .to_string();
+    tc->execute_search(search_term);
 }
 
 Result<std::string, lnav::console::user_message>
 execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
 {
-    db_label_source& dls = lnav_data.ld_db_row_source;
+    db_label_source& dls = *(ec.ec_label_source_stack.back());
     auto_mem<sqlite3_stmt> stmt(sqlite3_finalize);
     struct timeval start_tv, end_tv;
     std::string stmt_str = trim(sql);
     std::string retval;
     int retcode = SQLITE_OK;
 
-    log_info("Executing SQL: %s", sql.c_str());
+    if (lnav::sql::is_prql(stmt_str)) {
+        log_info("compiling PRQL: %s", stmt_str.c_str());
+
+#if HAVE_RUST_DEPS
+        auto opts = prqlc::Options{true, "sql.sqlite", true};
+
+        auto tree = sqlite_extension_prql;
+        for (const auto& mod : lnav_prql_modules) {
+            tree.emplace_back(prqlc::SourceTreeElement{
+                mod.get_name(),
+                mod.to_string_fragment().to_string(),
+            });
+        }
+        tree.emplace_back(prqlc::SourceTreeElement{"", stmt_str});
+        auto cr = prqlc::compile_tree(tree, opts);
+
+        for (const auto& msg : cr.messages) {
+            if (msg.kind != prqlc::MessageKind::Error) {
+                continue;
+            }
+
+            auto stmt_al = attr_line_t(stmt_str);
+            readline_sqlite_highlighter(stmt_al, 0);
+            auto um
+                = lnav::console::user_message::error(
+                      attr_line_t("unable to compile PRQL: ").append(stmt_al))
+                      .with_reason(
+                          attr_line_t::from_ansi_str((std::string) msg.reason));
+            if (!msg.display.empty()) {
+                um.with_note(
+                    attr_line_t::from_ansi_str((std::string) msg.display));
+            }
+            for (const auto& hint : msg.hints) {
+                um.with_help(attr_line_t::from_ansi_str((std::string) hint));
+                break;
+            }
+            return Err(um);
+        }
+        stmt_str = (std::string) cr.output;
+#else
+        auto um = lnav::console::user_message::error(
+            attr_line_t("PRQL is not supported in this build"));
+        return Err(um);
+#endif
+    }
+
+    log_info("Executing SQL: %s", stmt_str.c_str());
 
     auto old_mode = lnav_data.ld_mode;
     lnav_data.ld_mode = ln_mode_t::BUSY;
@@ -281,8 +321,9 @@ execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
         std::vector<std::string> args;
         split_ws(stmt_str, args);
 
-        auto* sql_cmd_map = injector::get<readline_context::command_map_t*,
-                                          sql_cmd_map_tag>();
+        const auto* sql_cmd_map
+            = injector::get<readline_context::command_map_t*,
+                            sql_cmd_map_tag>();
         auto cmd_iter = sql_cmd_map->find(args[0]);
 
         if (cmd_iter != sql_cmd_map->end()) {
@@ -294,10 +335,6 @@ execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
         }
     }
 
-    if (stmt_str == ".msgformats") {
-        stmt_str = MSG_FORMAT_STMT;
-    }
-
     ec.ec_accumulator->clear();
 
     const auto& source = ec.ec_source.back();
@@ -324,7 +361,8 @@ execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
             auto um = lnav::console::user_message::error(
                           "failed to compile SQL statement")
                           .with_reason(errmsg)
-                          .with_snippets(ec.ec_source);
+                          .with_snippets(ec.ec_source)
+                          .move();
 
             auto annotated_sql = annotate_sql_with_error(
                 lnav_data.ld_db.in(), curr_stmt, tail);
@@ -358,27 +396,19 @@ execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
         bool done = false;
 
         auto bound_values = TRY(bind_sql_parameters(ec, stmt.in()));
-        if (lnav_data.ld_rl_view != nullptr) {
-            if (lnav_data.ld_rl_view) {
-                lnav_data.ld_rl_view->set_attr_value(
-                    lnav::console::user_message::info(
-                        attr_line_t("executing SQL statement, press ")
-                            .append("CTRL+]"_hotkey)
-                            .append(" to cancel"))
-                        .to_attr_line());
-                lnav_data.ld_rl_view->do_update();
-            }
-        }
-
         ec.ec_sql_callback(ec, stmt.in());
         while (!done) {
             retcode = sqlite3_step(stmt.in());
 
             switch (retcode) {
                 case SQLITE_OK:
-                case SQLITE_DONE:
+                case SQLITE_DONE: {
+                    auto changes = sqlite3_changes(lnav_data.ld_db.in());
+
+                    log_info("sqlite3_changes() -> %d", changes);
                     done = true;
                     break;
+                }
 
                 case SQLITE_ROW:
                     ec.ec_sql_callback(ec, stmt.in());
@@ -414,8 +444,9 @@ execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
 
                     log_error("sqlite3_step error code: %d", retcode);
                     auto um = sqlite3_error_to_user_message(lnav_data.ld_db)
-                                  .with_snippets(ec.ec_source)
-                                  .with_note(bound_note);
+                                  .with_context_snippets(ec.ec_source)
+                                  .with_note(bound_note)
+                                  .move();
 
                     return Err(um);
                 }
@@ -437,14 +468,14 @@ execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
         }
         lnav_data.ld_filter_view.reload_data();
         lnav_data.ld_files_view.reload_data();
-        lnav_data.ld_views[LNV_DB].reload_data();
-        lnav_data.ld_views[LNV_DB].set_left(0);
 
         lnav_data.ld_active_files.fc_files
             | lnav::itertools::for_each(&logfile::dump_stats);
         if (ec.ec_sql_callback != sql_callback) {
             retval = ec.ec_accumulator->get_string();
         } else if (!dls.dls_rows.empty()) {
+            lnav_data.ld_views[LNV_DB].reload_data();
+            lnav_data.ld_views[LNV_DB].set_left(0);
             if (lnav_data.ld_flags & LNF_HEADLESS) {
                 if (ec.ec_local_vars.size() == 1) {
                     ensure_view(&lnav_data.ld_views[LNV_DB]);
@@ -501,6 +532,7 @@ execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
 
             if (lnav_data.ld_flags & LNF_HEADLESS) {
                 if (ec.ec_local_vars.size() == 1) {
+                    lnav_data.ld_views[LNV_DB].reload_data();
                     ensure_view(&lnav_data.ld_views[LNV_DB]);
                 }
             }
@@ -511,15 +543,74 @@ execute_sql(exec_context& ec, const std::string& sql, std::string& alt_msg)
     return Ok(retval);
 }
 
-static Result<std::string, lnav::console::user_message>
-execute_file_contents(exec_context& ec,
-                      const ghc::filesystem::path& path,
-                      bool multiline)
+Result<void, lnav::console::user_message>
+multiline_executor::push_back(string_fragment line)
 {
-    static ghc::filesystem::path stdin_path("-");
-    static ghc::filesystem::path dev_stdin_path("/dev/stdin");
+    this->me_line_number += 1;
+
+    if (line.trim().empty()) {
+        if (this->me_cmdline) {
+            this->me_cmdline = this->me_cmdline.value() + "\n";
+        }
+        return Ok();
+    }
+    if (line[0] == '#') {
+        return Ok();
+    }
+
+    switch (line[0]) {
+        case ':':
+        case '/':
+        case ';':
+        case '|':
+            if (this->me_cmdline) {
+                this->me_last_result
+                    = TRY(execute_from_file(this->me_exec_context,
+                                            this->me_source,
+                                            this->me_starting_line_number,
+                                            trim(this->me_cmdline.value())));
+            }
+
+            this->me_starting_line_number = this->me_line_number;
+            this->me_cmdline = line.to_string();
+            break;
+        default:
+            if (this->me_cmdline) {
+                this->me_cmdline = fmt::format(
+                    FMT_STRING("{}{}"), this->me_cmdline.value(), line);
+            } else {
+                this->me_last_result = TRY(
+                    execute_from_file(this->me_exec_context,
+                                      this->me_source,
+                                      this->me_line_number,
+                                      fmt::format(FMT_STRING(":{}"), line)));
+            }
+            break;
+    }
+
+    return Ok();
+}
+
+Result<std::string, lnav::console::user_message>
+multiline_executor::final()
+{
+    if (this->me_cmdline) {
+        this->me_last_result
+            = TRY(execute_from_file(this->me_exec_context,
+                                    this->me_source,
+                                    this->me_starting_line_number,
+                                    trim(this->me_cmdline.value())));
+    }
+
+    return Ok(this->me_last_result);
+}
+
+static Result<std::string, lnav::console::user_message>
+execute_file_contents(exec_context& ec, const std::filesystem::path& path)
+{
+    static const std::filesystem::path stdin_path("-");
+    static const std::filesystem::path dev_stdin_path("/dev/stdin");
 
-    std::string retval;
     FILE* file;
 
     if (path == stdin_path || path == dev_stdin_path) {
@@ -527,59 +618,23 @@ execute_file_contents(exec_context& ec,
             return ec.make_error("stdin has already been consumed");
         }
         file = stdin;
-    } else if ((file = fopen(path.c_str(), "r")) == nullptr) {
+    } else if ((file = fopen(path.c_str(), "re")) == nullptr) {
         return ec.make_error("unable to open file");
     }
 
-    int line_number = 0, starting_line_number = 0;
     auto_mem<char> line;
     size_t line_max_size;
     ssize_t line_size;
-    nonstd::optional<std::string> cmdline;
+    multiline_executor me(ec, path.string());
 
+    ec.ec_local_vars.top()["0"] = path.string();
     ec.ec_path_stack.emplace_back(path.parent_path());
     exec_context::output_guard og(ec);
     while ((line_size = getline(line.out(), &line_max_size, file)) != -1) {
-        line_number += 1;
-
-        if (trim(line.in()).empty()) {
-            if (multiline && cmdline) {
-                cmdline = cmdline.value() + "\n";
-            }
-            continue;
-        }
-        if (line[0] == '#') {
-            continue;
-        }
-
-        switch (line[0]) {
-            case ':':
-            case '/':
-            case ';':
-            case '|':
-                if (cmdline) {
-                    retval = TRY(execute_from_file(
-                        ec, path, starting_line_number, trim(cmdline.value())));
-                }
-
-                starting_line_number = line_number;
-                cmdline = std::string(line);
-                break;
-            default:
-                if (multiline) {
-                    cmdline = fmt::format("{}{}", cmdline.value(), line.in());
-                } else {
-                    retval = TRY(execute_from_file(
-                        ec, path, line_number, fmt::format(":{}", line.in())));
-                }
-                break;
-        }
+        TRY(me.push_back(string_fragment::from_bytes(line.in(), line_size)));
     }
 
-    if (cmdline) {
-        retval = TRY(execute_from_file(
-            ec, path, starting_line_number, trim(cmdline.value())));
-    }
+    auto retval = TRY(me.final());
 
     if (file == stdin) {
         if (isatty(STDOUT_FILENO)) {
@@ -594,44 +649,50 @@ execute_file_contents(exec_context& ec,
 }
 
 Result<std::string, lnav::console::user_message>
-execute_file(exec_context& ec, const std::string& path_and_args, bool multiline)
+execute_file(exec_context& ec, const std::string& path_and_args)
 {
+    static const intern_string_t SRC = intern_string::lookup("cmdline");
+
     available_scripts scripts;
-    std::vector<std::string> split_args;
     std::string retval, msg;
     shlex lexer(path_and_args);
 
     log_info("Executing file: %s", path_and_args.c_str());
 
-    if (!lexer.split(split_args, ec.ec_local_vars.top())) {
-        return ec.make_error("unable to parse path");
+    auto split_args_res = lexer.split(scoped_resolver{&ec.ec_local_vars.top()});
+    if (split_args_res.isErr()) {
+        auto split_err = split_args_res.unwrapErr();
+        auto um = lnav::console::user_message::error(
+                      "unable to parse script command-line")
+                      .with_reason(split_err.te_msg)
+                      .with_snippet(lnav::console::snippet::from(
+                          SRC, lexer.to_attr_line(split_err)))
+                      .move();
+
+        return Err(um);
     }
+    auto split_args = split_args_res.unwrap();
     if (split_args.empty()) {
         return ec.make_error("no script specified");
     }
 
     ec.ec_local_vars.push({});
 
-    auto script_name = split_args[0];
+    auto script_name = split_args[0].se_value;
     auto& vars = ec.ec_local_vars.top();
-    char env_arg_name[32];
     std::string star, open_error = "file not found";
 
     add_ansi_vars(vars);
 
-    snprintf(
-        env_arg_name, sizeof(env_arg_name), "%d", (int) split_args.size() - 1);
-
-    vars["#"] = env_arg_name;
+    vars["#"] = fmt::to_string(split_args.size() - 1);
     for (size_t lpc = 0; lpc < split_args.size(); lpc++) {
-        snprintf(env_arg_name, sizeof(env_arg_name), "%lu", lpc);
-        vars[env_arg_name] = split_args[lpc];
+        vars[fmt::to_string(lpc)] = split_args[lpc].se_value;
     }
     for (size_t lpc = 1; lpc < split_args.size(); lpc++) {
         if (lpc > 1) {
             star.append(" ");
         }
-        star.append(split_args[lpc]);
+        star.append(split_args[lpc].se_value);
     }
     vars["__all__"] = star;
 
@@ -642,24 +703,59 @@ execute_file(exec_context& ec, const std::string& path_and_args, bool multiline)
     if (iter != scripts.as_scripts.end()) {
         paths_to_exec = iter->second;
     }
+    if (is_url(script_name)) {
+        auto_mem<CURLU> cu(curl_url_cleanup);
+        cu = curl_url();
+        auto set_rc = curl_url_set(cu, CURLUPART_URL, script_name.c_str(), 0);
+        if (set_rc == CURLUE_OK) {
+            auto_mem<char> scheme_part(curl_free);
+            auto get_rc
+                = curl_url_get(cu, CURLUPART_SCHEME, scheme_part.out(), 0);
+            if (get_rc == CURLUE_OK
+                && string_fragment::from_c_str(scheme_part.in()) == "file")
+            {
+                auto_mem<char> path_part;
+                auto get_rc
+                    = curl_url_get(cu, CURLUPART_PATH, path_part.out(), 0);
+                if (get_rc == CURLUE_OK) {
+                    auto rp_res = lnav::filesystem::realpath(path_part.in());
+                    if (rp_res.isOk()) {
+                        struct script_metadata meta;
+
+                        meta.sm_path = rp_res.unwrap();
+                        extract_metadata_from_file(meta);
+                        paths_to_exec.push_back(meta);
+                    }
+                }
+            }
+        }
+    }
     if (script_name == "-" || script_name == "/dev/stdin") {
         paths_to_exec.push_back({script_name, "", "", ""});
     } else if (access(script_name.c_str(), R_OK) == 0) {
         struct script_metadata meta;
+        auto rp_res = lnav::filesystem::realpath(script_name);
 
-        meta.sm_path = script_name;
+        if (rp_res.isErr()) {
+            log_error("unable to get realpath() of %s -- %s",
+                      script_name.c_str(),
+                      rp_res.unwrapErr().c_str());
+            meta.sm_path = script_name;
+        } else {
+            meta.sm_path = rp_res.unwrap();
+        }
         extract_metadata_from_file(meta);
         paths_to_exec.push_back(meta);
     } else if (errno != ENOENT) {
         open_error = strerror(errno);
     } else {
-        auto script_path = ghc::filesystem::path(script_name);
+        auto script_path = std::filesystem::path(script_name);
 
         if (!script_path.is_absolute()) {
             script_path = ec.ec_path_stack.back() / script_path;
         }
 
-        if (ghc::filesystem::is_regular_file(script_path)) {
+        if (std::filesystem::is_regular_file(script_path)) {
             struct script_metadata meta;
 
             meta.sm_path = script_path;
@@ -672,8 +768,7 @@ execute_file(exec_context& ec, const std::string& path_and_args, bool multiline)
 
     if (!paths_to_exec.empty()) {
         for (auto& path_iter : paths_to_exec) {
-            retval
-                = TRY(execute_file_contents(ec, path_iter.sm_path, multiline));
+            retval = TRY(execute_file_contents(ec, path_iter.sm_path));
         }
     }
     ec.ec_local_vars.pop();
@@ -688,13 +783,13 @@ execute_file(exec_context& ec, const std::string& path_and_args, bool multiline)
 
 Result<std::string, lnav::console::user_message>
 execute_from_file(exec_context& ec,
-                  const ghc::filesystem::path& path,
+                  const std::string& src,
                   int line_number,
                   const std::string& cmdline)
 {
     std::string retval, alt_msg;
-    auto _sg = ec.enter_source(
-        intern_string::lookup(path.string()), line_number, cmdline);
+    auto _sg
+        = ec.enter_source(intern_string::lookup(src), line_number, cmdline);
 
     switch (cmdline[0]) {
         case ':':
@@ -715,10 +810,8 @@ execute_from_file(exec_context& ec,
             break;
     }
 
-    log_info("%s:%d:execute result -- %s",
-             path.c_str(),
-             line_number,
-             retval.c_str());
+    log_info(
+        "%s:%d:execute result -- %s", src.c_str(), line_number, retval.c_str());
 
     return Ok(retval);
 }
@@ -726,14 +819,27 @@ execute_from_file(exec_context& ec,
 Result<std::string, lnav::console::user_message>
 execute_any(exec_context& ec, const std::string& cmdline_with_mode)
 {
+    if (cmdline_with_mode.empty()) {
+        auto um = lnav::console::user_message::error("empty command")
+                      .with_help(
+                          "a command should start with ':', ';', '/', '|' and "
+                          "followed by the operation to perform")
+                      .move();
+        if (!ec.ec_source.empty()) {
+            um.with_snippet(ec.ec_source.back());
+        }
+        return Err(um);
+    }
+
     std::string retval, alt_msg, cmdline = cmdline_with_mode.substr(1);
     auto _cleanup = finally([&ec] {
         if (ec.is_read_write() &&
             // only rebuild in a script or non-interactive mode so we don't
             // block the UI.
-            (lnav_data.ld_flags & LNF_HEADLESS || ec.ec_path_stack.size() > 1))
+            lnav_data.ld_flags & LNF_HEADLESS)
         {
             rescan_files();
+            wait_for_pipers(std::nullopt);
             rebuild_indexes_repeatedly();
         }
     });
@@ -771,7 +877,7 @@ execute_init_commands(
         return;
     }
 
-    nonstd::optional<exec_context::output_t> ec_out;
+    std::optional<exec_context::output_t> ec_out;
     auto_fd fd_copy;
 
     if (!(lnav_data.ld_flags & LNF_HEADLESS)) {
@@ -785,11 +891,13 @@ execute_init_commands(
                               "");
             return;
         }
+        fcntl(fileno(tmpout), F_SETFD, FD_CLOEXEC);
         fd_copy = auto_fd::dup_of(fileno(tmpout));
+        fd_copy.close_on_exec();
         ec_out = std::make_pair(tmpout.release(), fclose);
     }
 
-    auto& dls = lnav_data.ld_db_row_source;
+    auto& dls = *(ec.ec_label_source_stack.back());
     int option_index = 1;
 
     {
@@ -804,6 +912,8 @@ execute_init_commands(
 
             wait_for_children();
 
+            lnav_data.ld_view_stack.top() |
+                [&ec](auto* tc) { ec.ec_top_line = tc->get_selection(); };
             log_debug("init cmd: %s", cmd.c_str());
             {
                 auto _sg
@@ -828,9 +938,18 @@ execute_init_commands(
                 }
 
                 rescan_files();
+                auto deadline = ui_clock::now();
+                if (lnav_data.ld_flags & LNF_HEADLESS) {
+                    deadline += 5s;
+                } else {
+                    deadline += 500ms;
+                }
+                wait_for_pipers(deadline);
                 rebuild_indexes_repeatedly();
             }
-            if (dls.dls_rows.size() > 1) {
+            if (!dls.dls_headers.empty() && lnav_data.ld_view_stack.size() == 1)
+            {
+                lnav_data.ld_views[LNV_DB].reload_data();
                 ensure_view(LNV_DB);
             }
         }
@@ -842,15 +961,20 @@ execute_init_commands(
     if (ec_out && fstat(fd_copy, &st) != -1 && st.st_size > 0) {
         static const auto OUTPUT_NAME = std::string("Initial command output");
 
-        lnav_data.ld_active_files.fc_file_names[OUTPUT_NAME]
-            .with_fd(std::move(fd_copy))
-            .with_include_in_session(false)
-            .with_detect_format(false);
-        lnav_data.ld_files_to_front.emplace_back(OUTPUT_NAME, 0_vl);
+        auto create_piper_res = lnav::piper::create_looper(
+            OUTPUT_NAME, std::move(fd_copy), auto_fd{});
+        if (create_piper_res.isOk()) {
+            lnav_data.ld_active_files.fc_file_names[OUTPUT_NAME]
+                .with_piper(create_piper_res.unwrap())
+                .with_include_in_session(false)
+                .with_detect_format(false)
+                .with_init_location(0_vl);
+            lnav_data.ld_files_to_front.emplace_back(OUTPUT_NAME, 0_vl);
 
-        if (lnav_data.ld_rl_view != nullptr) {
-            lnav_data.ld_rl_view->set_alt_value(
-                HELP_MSG_1(X, "to close the file"));
+            if (lnav_data.ld_rl_view != nullptr) {
+                lnav_data.ld_rl_view->set_alt_value(
+                    HELP_MSG_1(X, "to close the file"));
+            }
         }
     }
 
@@ -860,54 +984,59 @@ execute_init_commands(
 int
 sql_callback(exec_context& ec, sqlite3_stmt* stmt)
 {
-    auto& dls = lnav_data.ld_db_row_source;
+    const auto& vc = view_colors::singleton();
+    auto& dls = *(ec.ec_label_source_stack.back());
+    int ncols = sqlite3_column_count(stmt);
 
     if (!sqlite3_stmt_busy(stmt)) {
         dls.clear();
 
+        for (int lpc = 0; lpc < ncols; lpc++) {
+            const int type = sqlite3_column_type(stmt, lpc);
+            std::string colname = sqlite3_column_name(stmt, lpc);
+
+            dls.push_header(colname, type, false);
+        }
         return 0;
     }
 
-    auto& chart = dls.dls_chart;
-    auto& vc = view_colors::singleton();
-    int ncols = sqlite3_column_count(stmt);
-    int row_number;
-    int lpc, retval = 0;
-    auto set_vars = false;
+    int retval = 0;
+    auto set_vars = dls.dls_rows.empty();
 
-    row_number = dls.dls_rows.size();
-    dls.dls_rows.resize(row_number + 1);
-    if (dls.dls_headers.empty()) {
-        for (lpc = 0; lpc < ncols; lpc++) {
+    if (dls.dls_rows.empty()) {
+        for (int lpc = 0; lpc < ncols; lpc++) {
             int type = sqlite3_column_type(stmt, lpc);
             std::string colname = sqlite3_column_name(stmt, lpc);
-            bool graphable;
 
-            graphable = ((type == SQLITE_INTEGER || type == SQLITE_FLOAT)
-                         && !binary_search(lnav_data.ld_db_key_names.begin(),
-                                           lnav_data.ld_db_key_names.end(),
-                                           colname));
-
-            dls.push_header(colname, type, graphable);
+            bool graphable = (type == SQLITE_INTEGER || type == SQLITE_FLOAT)
+                && !binary_search(lnav_data.ld_db_key_names.begin(),
+                                  lnav_data.ld_db_key_names.end(),
+                                  colname);
+            auto& hm = dls.dls_headers[lpc];
+            hm.hm_column_type = type;
+            hm.hm_graphable = graphable;
             if (graphable) {
                 auto name_for_ident_attrs = colname;
                 auto attrs = vc.attrs_for_ident(name_for_ident_attrs);
                 for (size_t attempt = 0;
-                     chart.attrs_in_use(attrs) && attempt < 3;
+                     hm.hm_chart.attrs_in_use(attrs) && attempt < 3;
                      attempt++)
                 {
                     name_for_ident_attrs += " ";
                     attrs = vc.attrs_for_ident(name_for_ident_attrs);
                 }
-                chart.with_attrs_for_ident(colname, attrs);
-                dls.dls_headers.back().hm_title_attrs = attrs;
+                hm.hm_chart.with_attrs_for_ident(colname, attrs);
+                hm.hm_title_attrs = attrs;
+                hm.hm_column_size = std::max(hm.hm_column_size, size_t{10});
             }
         }
-        set_vars = true;
     }
-    for (lpc = 0; lpc < ncols; lpc++) {
+
+    auto row_number = dls.dls_rows.size();
+    dls.dls_rows.resize(row_number + 1);
+    for (int lpc = 0; lpc < ncols; lpc++) {
         auto* raw_value = sqlite3_column_value(stmt, lpc);
-        auto value_type = sqlite3_value_type(raw_value);
+        const auto value_type = sqlite3_value_type(raw_value);
         scoped_value_t value;
         auto& hm = dls.dls_headers[lpc];
 
@@ -955,6 +1084,48 @@ sql_callback(exec_context& ec, sqlite3_stmt* stmt)
     return retval;
 }
 
+int
+internal_sql_callback(exec_context& ec, sqlite3_stmt* stmt)
+{
+    if (!sqlite3_stmt_busy(stmt)) {
+        return 0;
+    }
+
+    const int ncols = sqlite3_column_count(stmt);
+
+    auto& vars = ec.ec_local_vars.top();
+
+    for (int lpc = 0; lpc < ncols; lpc++) {
+        const char* column_name = sqlite3_column_name(stmt, lpc);
+
+        if (sql_ident_needs_quote(column_name)) {
+            continue;
+        }
+
+        auto* raw_value = sqlite3_column_value(stmt, lpc);
+        auto value_type = sqlite3_column_type(stmt, lpc);
+        scoped_value_t value;
+        switch (value_type) {
+            case SQLITE_INTEGER:
+                value = (int64_t) sqlite3_value_int64(raw_value);
+                break;
+            case SQLITE_FLOAT:
+                value = sqlite3_value_double(raw_value);
+                break;
+            case SQLITE_NULL:
+                value = null_value_t{};
+                break;
+            default:
+                value = std::string((const char*) sqlite3_value_text(raw_value),
+                                    sqlite3_value_bytes(raw_value));
+                break;
+        }
+        vars[column_name] = value;
+    }
+
+    return 0;
+}
+
 std::future<std::string>
 pipe_callback(exec_context& ec, const std::string& cmdline, auto_fd& fd)
 {
@@ -963,7 +1134,7 @@ pipe_callback(exec_context& ec, const std::string& cmdline, auto_fd& fd)
     if (out) {
         FILE* file = *out;
 
-        return std::async(std::launch::async, [&fd, file]() {
+        return std::async(std::launch::async, [fd = std::move(fd), file]() {
             char buffer[1024];
             ssize_t rc;
 
@@ -978,26 +1149,37 @@ pipe_callback(exec_context& ec, const std::string& cmdline, auto_fd& fd)
             return std::string();
         });
     }
-    auto tmp_fd
-        = lnav::filesystem::open_temp_file(
-              ghc::filesystem::temp_directory_path() / "lnav.out.XXXXXX")
-              .map([](auto pair) {
-                  ghc::filesystem::remove(pair.first);
+    std::error_code errc;
+    std::filesystem::create_directories(lnav::paths::workdir(), errc);
+    auto open_temp_res = lnav::filesystem::open_temp_file(lnav::paths::workdir()
+                                                          / "exec.XXXXXX");
+    if (open_temp_res.isErr()) {
+        return lnav::futures::make_ready_future(
+            fmt::format(FMT_STRING("error: cannot open temp file -- {}"),
+                        open_temp_res.unwrapErr()));
+    }
+
+    auto tmp_pair = open_temp_res.unwrap();
+
+    auto reader = std::thread(
+        [in_fd = std::move(fd), out_fd = std::move(tmp_pair.second)]() {
+            char buffer[1024];
+            ssize_t rc;
+
+            while ((rc = read(in_fd, buffer, sizeof(buffer))) > 0) {
+                write(out_fd, buffer, rc);
+            }
+        });
+    reader.detach();
 
-                  return std::move(pair.second);
-              })
-              .expect("Cannot create temporary file for callback");
-    auto pp
-        = std::make_shared<piper_proc>(std::move(fd), false, std::move(tmp_fd));
     static int exec_count = 0;
-
-    lnav_data.ld_pipers.push_back(pp);
     auto desc
-        = fmt::format(FMT_STRING("[{}] Output of {}"), exec_count++, cmdline);
-    lnav_data.ld_active_files.fc_file_names[desc]
-        .with_fd(pp->get_fd())
+        = fmt::format(FMT_STRING("exec-{}-output {}"), exec_count++, cmdline);
+    lnav_data.ld_active_files.fc_file_names[tmp_pair.first]
+        .with_filename(desc)
         .with_include_in_session(false)
-        .with_detect_format(false);
+        .with_detect_format(false)
+        .with_init_location(0_vl);
     lnav_data.ld_files_to_front.emplace_back(desc, 0_vl);
     if (lnav_data.ld_rl_view != nullptr) {
         lnav_data.ld_rl_view->set_alt_value(HELP_MSG_1(X, "to close the file"));
@@ -1013,7 +1195,7 @@ add_global_vars(exec_context& ec)
         shlex subber(iter.second);
         std::string str;
 
-        if (!subber.eval(str, ec.ec_global_vars)) {
+        if (!subber.eval(str, scoped_resolver{&ec.ec_global_vars})) {
             log_error("Unable to evaluate global variable value: %s",
                       iter.second.c_str());
             continue;
@@ -1047,12 +1229,13 @@ exec_context::clear_output()
             out.second(out.first);
         }
     };
-    this->ec_output_stack.back() = std::make_pair("default", nonstd::nullopt);
+    this->ec_output_stack.back() = std::make_pair("default", std::nullopt);
 }
 
-exec_context::exec_context(logline_value_vector* line_values,
-                           sql_callback_t sql_callback,
-                           pipe_callback_t pipe_callback)
+exec_context::
+exec_context(logline_value_vector* line_values,
+             sql_callback_t sql_callback,
+             pipe_callback_t pipe_callback)
     : ec_line_values(line_values),
       ec_accumulator(std::make_unique<attr_line_t>()),
       ec_sql_callback(sql_callback), ec_pipe_callback(pipe_callback)
@@ -1063,18 +1246,42 @@ exec_context::exec_context(logline_value_vector* line_values,
     this->ec_path_stack.emplace_back(".");
     this->ec_source.emplace_back(
         lnav::console::snippet::from(COMMAND_SRC, "").with_line(1));
-    this->ec_output_stack.emplace_back("screen", nonstd::nullopt);
+    this->ec_output_stack.emplace_back("screen", std::nullopt);
     this->ec_error_callback_stack.emplace_back(
         [](const auto& um) { lnav::console::print(stderr, um); });
 }
 
-void
+Result<std::string, lnav::console::user_message>
 exec_context::execute(const std::string& cmdline)
 {
+    if (this->get_provenance<mouse_input>()) {
+        require(!lnav_data.ld_rl_view->is_active());
+
+        int context = 0;
+        switch (cmdline[0]) {
+            case '/':
+                context = lnav::enums::to_underlying(ln_mode_t::SEARCH);
+                break;
+            case ':':
+                context = lnav::enums::to_underlying(ln_mode_t::COMMAND);
+                break;
+            case ';':
+                context = lnav::enums::to_underlying(ln_mode_t::SQL);
+                break;
+            case '|':
+                context = lnav::enums::to_underlying(ln_mode_t::EXEC);
+                break;
+        }
+
+        lnav_data.ld_rl_view->append_to_history(context, cmdline.substr(1));
+    }
+
     auto exec_res = execute_any(*this, cmdline);
     if (exec_res.isErr()) {
         this->ec_error_callback_stack.back()(exec_res.unwrapErr());
     }
+
+    return exec_res;
 }
 
 void
@@ -1104,6 +1311,11 @@ exec_context::add_error_context(lnav::console::user_message& um)
     }
 }
 
+exec_context::sql_callback_guard
+exec_context::push_callback(sql_callback_t cb)
+{
+    return sql_callback_guard(*this, cb);
+}
 exec_context::source_guard
 exec_context::enter_source(intern_string_t path,
                            int line_number,
@@ -1117,9 +1329,10 @@ exec_context::enter_source(intern_string_t path,
     return {this};
 }
 
-exec_context::output_guard::output_guard(exec_context& context,
-                                         std::string name,
-                                         const nonstd::optional<output_t>& file)
+exec_context::output_guard::
+output_guard(exec_context& context,
+             std::string name,
+             const std::optional<output_t>& file)
     : sg_context(context)
 {
     if (file) {
@@ -1128,8 +1341,28 @@ exec_context::output_guard::output_guard(exec_context& context,
     context.ec_output_stack.emplace_back(std::move(name), file);
 }
 
-exec_context::output_guard::~output_guard()
+exec_context::output_guard::~
+output_guard()
 {
     this->sg_context.clear_output();
     this->sg_context.ec_output_stack.pop_back();
 }
+exec_context::sql_callback_guard::
+sql_callback_guard(exec_context& context, sql_callback_t cb)
+    : scg_context(context), scg_old_callback(context.ec_sql_callback)
+{
+    context.ec_sql_callback = cb;
+}
+exec_context::sql_callback_guard::
+sql_callback_guard(sql_callback_guard&& other)
+    : scg_context(other.scg_context),
+      scg_old_callback(std::exchange(other.scg_old_callback, nullptr))
+{
+}
+exec_context::sql_callback_guard::~
+sql_callback_guard()
+{
+    if (this->scg_old_callback != nullptr) {
+        this->scg_context.ec_sql_callback = this->scg_old_callback;
+    }
+}
