@@ -106,7 +106,7 @@ Original code 2006 June 05 by relicoder.
 
 */
 
-//#include "config.h"
+// #include "config.h"
 
 // #define COMPILE_SQLITE_EXTENSIONS_AS_LOADABLE_MODULE 1
 #define HAVE_ACOSH     1
@@ -2550,6 +2550,7 @@ common_extension_functions(struct FuncDef** basic_funcs,
             reverseFunc,
             help_text("reverse")
                 .sql_function()
+                .with_prql_path({"text", "reverse"})
                 .with_summary("Returns the reverse of the given string.")
                 .with_parameter({"str", "The string to reverse."})
                 .with_tags({"string"})
@@ -2671,13 +2672,13 @@ common_extension_functions(struct FuncDef** basic_funcs,
 
     /* Aggregate functions */
     static struct FuncDefAgg aAggs[] = {
-        {"stdev", 1, 0, varianceStep, stdevFinalize},
-        {"stddev", 1, 0, varianceStep, stdevFinalize},
-        {"variance", 1, 0, varianceStep, varianceFinalize},
-        {"mode", 1, 0, modeStep, modeFinalize},
-        {"median", 1, 0, modeStep, medianFinalize},
-        {"lower_quartile", 1, 0, modeStep, lower_quartileFinalize},
-        {"upper_quartile", 1, 0, modeStep, upper_quartileFinalize},
+        {"stdev", 1, SQLITE_UTF8, 0, varianceStep, stdevFinalize},
+        {"stddev", 1, SQLITE_UTF8, 0, varianceStep, stdevFinalize},
+        {"variance", 1, SQLITE_UTF8, 0, varianceStep, varianceFinalize},
+        {"mode", 1, SQLITE_UTF8, 0, modeStep, modeFinalize},
+        {"median", 1, SQLITE_UTF8, 0, modeStep, medianFinalize},
+        {"lower_quartile", 1, SQLITE_UTF8, 0, modeStep, lower_quartileFinalize},
+        {"upper_quartile", 1, SQLITE_UTF8, 0, modeStep, upper_quartileFinalize},
 
         {nullptr},
     };
