@@ -37,7 +37,7 @@
 #include "lnav.hh"
 
 static auto TOGGLE_MSG = "Press " ANSI_BOLD("TAB") " to edit ";
-static auto EXIT_MSG = "Press " ANSI_BOLD("q") " to exit ";
+static auto EXIT_MSG = "Press " ANSI_BOLD("ESC") " to exit ";
 
 static auto CREATE_HELP = ANSI_BOLD("i") "/" ANSI_BOLD("o") ": Create in/out";
 static auto ENABLE_HELP = ANSI_BOLD("SPC") ": ";
@@ -47,13 +47,17 @@ static auto DELETE_HELP = ANSI_BOLD("D") ": Delete";
 static auto FILTERING_HELP = ANSI_BOLD("f") ": ";
 static auto JUMP_HELP = ANSI_BOLD("ENTER") ": Jump To";
 static auto CLOSE_HELP = ANSI_BOLD("X") ": Close";
+static auto FOCUS_DETAILS_HELP = ANSI_BOLD("CTRL-]") ": Focus on details view";
 
-filter_status_source::filter_status_source()
+filter_status_source::
+filter_status_source()
 {
     this->tss_fields[TSF_TITLE].set_width(14);
     this->tss_fields[TSF_TITLE].set_role(role_t::VCR_STATUS_TITLE);
     this->tss_fields[TSF_TITLE].set_value(" " ANSI_ROLE("T") "ext Filters ",
                                           role_t::VCR_STATUS_TITLE_HOTKEY);
+    this->tss_fields[TSF_TITLE].on_click
+        = [](status_field&) { set_view_mode(ln_mode_t::FILTER); };
 
     this->tss_fields[TSF_STITCH_TITLE].set_width(2);
     this->tss_fields[TSF_STITCH_TITLE].set_stitch_value(
@@ -73,6 +77,8 @@ filter_status_source::filter_status_source()
         role_t::VCR_STATUS_DISABLED_TITLE);
     this->tss_fields[TSF_FILES_TITLE].set_value(" " ANSI_ROLE("F") "iles ",
                                                 role_t::VCR_STATUS_HOTKEY);
+    this->tss_fields[TSF_FILES_TITLE].on_click
+        = [](status_field&) { set_view_mode(ln_mode_t::FILES); };
 
     this->tss_fields[TSF_FILES_RIGHT_STITCH].set_width(2);
     this->tss_fields[TSF_FILES_RIGHT_STITCH].set_stitch_value(
@@ -98,6 +104,7 @@ filter_status_source::statusview_fields()
             break;
         case ln_mode_t::FILTER:
         case ln_mode_t::FILES:
+        case ln_mode_t::FILE_DETAILS:
             this->tss_fields[TSF_HELP].set_value(EXIT_MSG);
             break;
         default:
@@ -106,6 +113,7 @@ filter_status_source::statusview_fields()
     }
 
     if (lnav_data.ld_mode == ln_mode_t::FILES
+        || lnav_data.ld_mode == ln_mode_t::FILE_DETAILS
         || lnav_data.ld_mode == ln_mode_t::SEARCH_FILES)
     {
         this->tss_fields[TSF_FILES_TITLE].set_value(
@@ -122,7 +130,8 @@ filter_status_source::statusview_fields()
     } else {
         this->tss_fields[TSF_FILES_TITLE].set_value(" " ANSI_ROLE("F") "iles ",
                                                     role_t::VCR_STATUS_HOTKEY);
-        if (lnav_data.ld_active_files.fc_name_to_errors.empty()) {
+        if (lnav_data.ld_active_files.fc_name_to_errors->readAccess()->empty())
+        {
             this->tss_fields[TSF_FILES_TITLE].set_role(
                 role_t::VCR_STATUS_DISABLED_TITLE);
         } else {
@@ -130,12 +139,13 @@ filter_status_source::statusview_fields()
                 role_t::VCR_ALERT_STATUS);
 
             auto& fc = lnav_data.ld_active_files;
-            if (fc.fc_name_to_errors.size() == 1) {
+            if (fc.fc_name_to_errors->readAccess()->size() == 1) {
                 this->tss_error.set_value(" error: a file cannot be opened ");
             } else {
                 this->tss_error.set_value(
                     " error: %u files cannot be opened ",
-                    lnav_data.ld_active_files.fc_name_to_errors.size());
+                    lnav_data.ld_active_files.fc_name_to_errors->readAccess()
+                        ->size());
             }
         }
         this->tss_fields[TSF_FILES_RIGHT_STITCH].set_stitch_value(
@@ -181,7 +191,7 @@ status_field&
 filter_status_source::statusview_value_for_field(int field)
 {
     if (field == TSF_FILTERED
-        && !lnav_data.ld_active_files.fc_name_to_errors.empty())
+        && !lnav_data.ld_active_files.fc_name_to_errors->readAccess()->empty())
     {
         return this->tss_error;
     }
@@ -225,7 +235,8 @@ filter_status_source::update_filtered(text_sub_source* tss)
     }
 }
 
-filter_help_status_source::filter_help_status_source()
+filter_help_status_source::
+filter_help_status_source()
 {
     this->fss_help.set_min_width(10);
     this->fss_help.set_share(1);
@@ -286,10 +297,11 @@ filter_help_status_source::statusview_fields()
                     tss->tss_apply_filters ? "Disable Filtering"
                                            : "Enable Filtering");
             }
-        } else if (lnav_data.ld_mode == ln_mode_t::FILES
+        } else if ((lnav_data.ld_mode == ln_mode_t::FILES
+                    || lnav_data.ld_mode == ln_mode_t::FILE_DETAILS)
                    && lnav_data.ld_session_loaded)
         {
-            auto& lv = lnav_data.ld_files_view;
+            const auto& lv = lnav_data.ld_files_view;
             auto sel = files_model::from_selection(lv.get_selection());
 
             sel.match(
@@ -307,9 +319,16 @@ filter_help_status_source::statusview_fields()
                     if (ld_opt && !ld_opt.value()->ld_visible) {
                         vis_help = "Show";
                     }
+                    const auto* focus_details_help
+                        = lnav_data.ld_mode == ln_mode_t::FILES
+                        ? FOCUS_DETAILS_HELP
+                        : "";
 
-                    this->fss_help.set_value(
-                        "  %s%s  %s", ENABLE_HELP, vis_help, JUMP_HELP);
+                    this->fss_help.set_value("  %s%s  %s  %s",
+                                             ENABLE_HELP,
+                                             vis_help,
+                                             JUMP_HELP,
+                                             focus_details_help);
                 });
         }
     };
