@@ -57,8 +57,10 @@
 #include "base/math_util.hh"
 #include "base/paths.hh"
 #include "fmtlib/fmt/format.h"
+#include "hasher.hh"
 #include "line_buffer.hh"
-#include "lnav_util.hh"
+#include "piper.looper.hh"
+#include "scn/scn.h"
 
 using namespace std::chrono_literals;
 
@@ -133,9 +135,10 @@ private:
 
 #define Z_BUFSIZE      65536U
 #define SYNCPOINT_SIZE (1024 * 1024)
-line_buffer::gz_indexed::gz_indexed()
+line_buffer::gz_indexed::
+gz_indexed()
 {
-    if ((this->inbuf = (Bytef*) malloc(Z_BUFSIZE)) == NULL) {
+    if ((this->inbuf = auto_mem<Bytef>::malloc(Z_BUFSIZE)) == nullptr) {
         throw std::bad_alloc();
     }
 }
@@ -160,13 +163,8 @@ line_buffer::gz_indexed::init_stream()
     }
 
     // initialize inflate struct
-    this->strm.zalloc = Z_NULL;
-    this->strm.zfree = Z_NULL;
-    this->strm.opaque = Z_NULL;
+    int rc = inflateInit2(&this->strm, GZ_HEADER_MODE);
     this->strm.avail_in = 0;
-    this->strm.next_in = Z_NULL;
-    this->strm.avail_out = 0;
-    int rc = inflateInit2(&strm, GZ_HEADER_MODE);
     if (rc != Z_OK) {
         throw(rc);  // FIXME: exception wrapper
     }
@@ -191,7 +189,7 @@ line_buffer::gz_indexed::continue_stream()
 }
 
 void
-line_buffer::gz_indexed::open(int fd, header_data& hd)
+line_buffer::gz_indexed::open(int fd, lnav::gzip::header& hd)
 {
     this->close();
     this->init_stream();
@@ -225,7 +223,6 @@ line_buffer::gz_indexed::open(int fd, header_data& hd)
 
             inflate(&this->strm, Z_BLOCK);
             inflateEnd(&this->strm);
-
             this->strm.next_out = Z_NULL;
             this->strm.next_in = Z_NULL;
             this->strm.next_in = Z_NULL;
@@ -238,9 +235,17 @@ line_buffer::gz_indexed::open(int fd, header_data& hd)
                     log_debug("%d: no gzip header data", fd);
                     break;
                 case 1:
-                    hd.hd_mtime.tv_sec = gz_hd.time;
-                    hd.hd_name = std::string((char*) name);
-                    hd.hd_comment = std::string((char*) comment);
+                    hd.h_mtime.tv_sec = gz_hd.time;
+                    hd.h_name = std::string((char*) name);
+                    hd.h_comment = std::string((char*) comment);
+                    log_info(
+                        "%d: read gzip header (mtime=%d; name='%s'; "
+                        "comment='%s'; crc=%x)",
+                        fd,
+                        hd.h_mtime.tv_sec,
+                        hd.h_name.c_str(),
+                        hd.h_comment.c_str(),
+                        gz_hd.hcrc);
                     break;
                 default:
                     log_error("%d: failed to read gzip header data", fd);
@@ -281,7 +286,8 @@ line_buffer::gz_indexed::stream_data(void* buf, size_t size)
                 // stream
                 continue_stream();
             } else if (err != Z_OK) {
-                log_error(" inflate-error: %d  %s",
+                log_error(" inflate-error at %d: %d  %s",
+                          this->strm.total_in,
                           (int) err,
                           this->strm.msg ? this->strm.msg : "");
                 break;
@@ -362,13 +368,19 @@ line_buffer::gz_indexed::read(void* buf, size_t offset, size_t size)
     return bytes;
 }
 
-line_buffer::line_buffer()
+line_buffer::
+line_buffer()
 {
     ensure(this->invariant());
 }
 
-line_buffer::~line_buffer()
+line_buffer::~
+line_buffer()
 {
+    if (this->lb_loader_future.valid()) {
+        this->lb_loader_future.wait();
+    }
+
     auto empty_fd = auto_fd();
 
     // Make sure any shared refs take ownership of the data.
@@ -408,7 +420,33 @@ line_buffer::set_fd(auto_fd& fd)
             char gz_id[2 + 1 + 1 + 4];
 
             if (pread(fd, gz_id, sizeof(gz_id), 0) == sizeof(gz_id)) {
-                if (gz_id[0] == '\037' && gz_id[1] == '\213') {
+                auto piper_hdr_opt = lnav::piper::read_header(fd, gz_id);
+
+                if (piper_hdr_opt) {
+                    static intern_string_t SRC = intern_string::lookup("piper");
+
+                    auto meta_buf = std::move(piper_hdr_opt.value());
+
+                    auto meta_sf = string_fragment::from_bytes(meta_buf.in(),
+                                                               meta_buf.size());
+                    auto meta_parse_res
+                        = lnav::piper::header_handlers.parser_for(SRC).of(
+                            meta_sf);
+                    if (meta_parse_res.isErr()) {
+                        log_error("failed to parse piper header: %s",
+                                  meta_parse_res.unwrapErr()[0]
+                                      .to_attr_line()
+                                      .get_string()
+                                      .c_str());
+                        throw error(EINVAL);
+                    }
+
+                    this->lb_line_metadata = true;
+                    this->lb_file_offset
+                        = lnav::piper::HEADER_SIZE + meta_buf.size();
+                    this->lb_piper_header_size = this->lb_file_offset;
+                    this->lb_header = meta_parse_res.unwrap();
+                } else if (gz_id[0] == '\037' && gz_id[1] == '\213') {
                     int gzfd = dup(fd);
 
                     log_perror(fcntl(gzfd, F_SETFD, FD_CLOEXEC));
@@ -416,14 +454,19 @@ line_buffer::set_fd(auto_fd& fd)
                         close(gzfd);
                         throw error(errno);
                     }
-                    this->lb_gz_file.writeAccess()->open(gzfd, this->lb_header);
+                    lnav::gzip::header hdr;
+
+                    this->lb_gz_file.writeAccess()->open(gzfd, hdr);
                     this->lb_compressed = true;
-                    this->lb_file_time = this->lb_header.hd_mtime.tv_sec;
+                    this->lb_file_time = hdr.h_mtime.tv_sec;
                     if (this->lb_file_time < 0) {
                         this->lb_file_time = 0;
                     }
                     this->lb_compressed_offset
                         = lseek(this->lb_fd, 0, SEEK_CUR);
+                    if (!hdr.empty()) {
+                        this->lb_header = std::move(hdr);
+                    }
                     this->resize_buffer(INITIAL_COMPRESSED_BUFFER_SIZE);
                 }
 #ifdef HAVE_BZLIB_H
@@ -735,13 +778,12 @@ line_buffer::fill_range(file_off_t start, ssize_t max_length)
                   start,
                   this->lb_loader_file_offset.value());
 #endif
-        nonstd::optional<std::chrono::system_clock::time_point> wait_start;
+        std::optional<std::chrono::system_clock::time_point> wait_start;
 
         if (this->lb_loader_future.wait_for(std::chrono::seconds(0))
             != std::future_status::ready)
         {
-            wait_start
-                = nonstd::make_optional(std::chrono::system_clock::now());
+            wait_start = std::make_optional(std::chrono::system_clock::now());
         }
         retval = this->lb_loader_future.get();
         if (false && wait_start) {
@@ -752,7 +794,7 @@ line_buffer::fill_range(file_off_t start, ssize_t max_length)
         this->lb_loader_future = {};
         this->lb_share_manager.invalidate_refs();
         this->lb_file_offset = this->lb_loader_file_offset.value();
-        this->lb_loader_file_offset = nonstd::nullopt;
+        this->lb_loader_file_offset = std::nullopt;
         this->lb_buffer.swap(this->lb_alt_buffer.value());
         this->lb_alt_buffer.value().clear();
         this->lb_line_starts = std::move(this->lb_alt_line_starts);
@@ -763,7 +805,9 @@ line_buffer::fill_range(file_off_t start, ssize_t max_length)
         this->lb_alt_line_has_ansi.clear();
         this->lb_stats.s_used_preloads += 1;
     }
-    if (this->in_range(start) && this->in_range(start + max_length - 1)) {
+    if (this->in_range(start)
+        && (max_length == 0 || this->in_range(start + max_length - 1)))
+    {
         /* Cache already has the data, nothing to do. */
         retval = true;
         if (!lnav::pid::in_child && this->lb_seekable && this->lb_buffer.full()
@@ -1022,11 +1066,16 @@ line_buffer::fill_range(file_off_t start, ssize_t max_length)
 Result<line_info, std::string>
 line_buffer::load_next_line(file_range prev_line)
 {
+    const char* line_start = nullptr;
     bool done = false;
     line_info retval;
 
     require(this->lb_fd != -1);
 
+    if (this->lb_line_metadata && prev_line.fr_offset == 0) {
+        prev_line.fr_offset = this->lb_piper_header_size;
+    }
+
     auto offset = prev_line.next_offset();
     ssize_t request_size = INITIAL_REQUEST_SIZE;
     retval.li_file_range.fr_offset = offset;
@@ -1044,9 +1093,17 @@ line_buffer::load_next_line(file_range prev_line)
             return Ok(retval);
         }
     }
+    if (prev_line.next_offset() == 0) {
+        auto is_utf_res = is_utf8(string_fragment::from_bytes(
+            this->lb_buffer.begin(), this->lb_buffer.size()));
+        this->lb_is_utf8 = is_utf_res.is_valid();
+        if (!this->lb_is_utf8) {
+            log_warning("input is not utf8 -- %s", is_utf_res.usr_message);
+        }
+    }
     while (!done) {
         auto old_retval_size = retval.li_file_range.fr_size;
-        const char *line_start, *lf = nullptr;
+        const char* lf = nullptr;
 
         /* Find the data in the cache and */
         line_start = this->get_range(offset, retval.li_file_range.fr_size);
@@ -1177,16 +1234,40 @@ line_buffer::load_next_line(file_range prev_line)
         = retval.li_utf8_scan_result.usr_has_ansi;
     retval.li_file_range.fr_metadata.m_valid_utf
         = retval.li_utf8_scan_result.is_valid();
+
+    if (this->lb_line_metadata) {
+        auto sv = scn::string_view{
+            line_start,
+            (size_t) retval.li_file_range.fr_size,
+        };
+
+        char level = '\0';
+        auto scan_res = scn::scan(sv,
+                                  "{}.{}:{};",
+                                  retval.li_timestamp.tv_sec,
+                                  retval.li_timestamp.tv_usec,
+                                  level);
+        if (scan_res) {
+            retval.li_timestamp.tv_sec
+                = lnav::to_local_time(date::sys_seconds{std::chrono::seconds{
+                                          retval.li_timestamp.tv_sec}})
+                      .time_since_epoch()
+                      .count();
+            retval.li_level = abbrev2level(&level, 1);
+        }
+    }
+
     return Ok(retval);
 }
 
 Result<shared_buffer_ref, std::string>
-line_buffer::read_range(const file_range fr)
+line_buffer::read_range(file_range fr)
 {
     shared_buffer_ref retval;
     const char* line_start;
     file_ssize_t avail;
 
+#if 0
     if (this->lb_last_line_offset != -1
         && fr.fr_offset > this->lb_last_line_offset)
     {
@@ -1200,6 +1281,7 @@ line_buffer::read_range(const file_range fr)
                         fr.fr_offset,
                         this->lb_last_line_offset));
     }
+#endif
 
     if (!(this->in_range(fr.fr_offset)
           && this->in_range(fr.fr_offset + fr.fr_size - 1)))
@@ -1214,6 +1296,15 @@ line_buffer::read_range(const file_range fr)
         return Err(fmt::format(
             FMT_STRING("short-read (need: {}; avail: {})"), fr.fr_size, avail));
     }
+    if (this->lb_line_metadata) {
+        auto new_start
+            = static_cast<const char*>(memchr(line_start, ';', fr.fr_size));
+        if (new_start) {
+            auto offset = new_start - line_start + 1;
+            line_start += offset;
+            fr.fr_size -= offset;
+        }
+    }
     retval.share(this->lb_share_manager, line_start, fr.fr_size);
     retval.get_metadata() = fr.fr_metadata;
 
@@ -1227,8 +1318,8 @@ line_buffer::get_available()
             static_cast<file_ssize_t>(this->lb_buffer.size())};
 }
 
-line_buffer::gz_indexed::indexDict::indexDict(const z_stream& s,
-                                              const file_size_t size)
+line_buffer::gz_indexed::indexDict::
+indexDict(const z_stream& s, const file_size_t size)
 {
     assert((s.data_type & GZ_END_OF_BLOCK_MASK));
     assert(!(s.data_type & GZ_END_OF_FILE_MASK));
@@ -1289,7 +1380,7 @@ line_buffer::quiesce()
     }
 }
 
-static ghc::filesystem::path
+static std::filesystem::path
 line_buffer_cache_path()
 {
     return lnav::paths::workdir() / "buffer-cache";
@@ -1320,7 +1411,7 @@ line_buffer::enable_cache()
                                 .to_string();
     auto cache_dir = line_buffer_cache_path() / cached_base_name.substr(0, 2);
 
-    ghc::filesystem::create_directories(cache_dir);
+    std::filesystem::create_directories(cache_dir);
 
     auto cached_file_name = fmt::format(FMT_STRING("{}.bin"), cached_base_name);
     auto cached_file_path = cache_dir / cached_file_name;
@@ -1333,14 +1424,14 @@ line_buffer::enable_cache()
     auto fl = lnav::filesystem::file_lock(cached_file_path);
     auto guard = lnav::filesystem::file_lock::guard(&fl);
 
-    if (ghc::filesystem::exists(cached_done_path)) {
-        log_info("%d:using existing cache file");
+    if (std::filesystem::exists(cached_done_path)) {
+        log_info("%d:using existing cache file", this->lb_fd.get());
         auto open_res = lnav::filesystem::open_file(cached_file_path, O_RDWR);
         if (open_res.isOk()) {
             this->lb_cached_fd = open_res.unwrap();
             return;
         }
-        ghc::filesystem::remove(cached_done_path);
+        std::filesystem::remove(cached_done_path);
     }
 
     auto create_res = lnav::filesystem::create_file(
@@ -1355,7 +1446,7 @@ line_buffer::enable_cache()
     auto write_fd = create_res.unwrap();
     auto done = false;
 
-    static const ssize_t FILL_LENGTH = 1024 * 1024;
+    static constexpr ssize_t FILL_LENGTH = 1024 * 1024;
     auto off = file_off_t{0};
     while (!done) {
         log_debug("%d: caching file content at %d", this->lb_fd.get(), off);
@@ -1385,17 +1476,18 @@ void
 line_buffer::cleanup_cache()
 {
     (void) std::async(std::launch::async, []() {
-        auto now = std::chrono::system_clock::now();
+        auto now = std::filesystem::file_time_type::clock::now();
         auto cache_path = line_buffer_cache_path();
-        std::vector<ghc::filesystem::path> to_remove;
+        std::vector<std::filesystem::path> to_remove;
+        std::error_code ec;
 
         for (const auto& cache_subdir :
-             ghc::filesystem::directory_iterator(cache_path))
+             std::filesystem::directory_iterator(cache_path, ec))
         {
             for (const auto& entry :
-                 ghc::filesystem::directory_iterator(cache_subdir))
+                 std::filesystem::directory_iterator(cache_subdir, ec))
             {
-                auto mtime = ghc::filesystem::last_write_time(entry.path());
+                auto mtime = std::filesystem::last_write_time(entry.path());
                 auto exp_time = mtime + 1h;
                 if (now < exp_time) {
                     continue;
@@ -1407,7 +1499,7 @@ line_buffer::cleanup_cache()
 
         for (auto& entry : to_remove) {
             log_debug("removing compressed file cache: %s", entry.c_str());
-            ghc::filesystem::remove_all(entry);
+            std::filesystem::remove_all(entry, ec);
         }
     });
 }
