@@ -27,18 +27,28 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <queue>
-
 #include "lnav.management_cli.hh"
 
+#include <glob.h>
+
+#include "base/fs_util.hh"
+#include "base/humanize.hh"
+#include "base/humanize.time.hh"
 #include "base/itertools.hh"
+#include "base/paths.hh"
 #include "base/result.h"
 #include "base/string_util.hh"
+#include "crashd.client.hh"
+#include "file_options.hh"
+#include "fmt/chrono.h"
 #include "fmt/format.h"
 #include "itertools.similar.hh"
+#include "lnav.hh"
+#include "lnav_config.hh"
 #include "log_format.hh"
 #include "log_format_ext.hh"
 #include "mapbox/variant.hpp"
+#include "piper.looper.hh"
 #include "regex101.import.hh"
 #include "session_data.hh"
 
@@ -52,21 +62,139 @@ struct no_subcmd_t {
     CLI::App* ns_root_app{nullptr};
 };
 
+static auto DEFAULT_WRAPPING
+    = text_wrap_settings{}.with_padding_indent(4).with_width(60);
+
 inline attr_line_t&
 symbol_reducer(const std::string& elem, attr_line_t& accum)
 {
-    return accum.append("\n   ").append(lnav::roles::symbol(elem));
+    if (!accum.empty()) {
+        accum.append(", ");
+    }
+    return accum.append(lnav::roles::symbol(elem));
 }
 
 inline attr_line_t&
 subcmd_reducer(const CLI::App* app, attr_line_t& accum)
 {
-    return accum.append("\n \u2022 ")
+    return accum.append("\n ")
+        .append("\u2022"_list_glyph)
+        .append(" ")
         .append(lnav::roles::keyword(app->get_name()))
         .append(": ")
         .append(app->get_description());
 }
 
+struct subcmd_config_t {
+    using action_t = std::function<perform_result_t(const subcmd_config_t&)>;
+
+    CLI::App* sc_config_app{nullptr};
+    action_t sc_action;
+    std::string sc_path;
+
+    static perform_result_t default_action(const subcmd_config_t& sc)
+    {
+        auto um
+            = console::user_message::error(
+                  "expecting an operation related to the regex101.com "
+                  "integration")
+                  .with_help(sc.sc_config_app->get_subcommands({})
+                             | lnav::itertools::fold(
+                                 subcmd_reducer,
+                                 attr_line_t{"the available operations are:"}))
+                  .move();
+
+        return {std::move(um)};
+    }
+
+    static perform_result_t get_action(const subcmd_config_t&)
+    {
+        auto config_str = dump_config();
+        auto um = console::user_message::raw(config_str);
+
+        return {std::move(um)};
+    }
+
+    static perform_result_t blame_action(const subcmd_config_t&)
+    {
+        auto blame = attr_line_t();
+
+        for (const auto& pair : lnav_config_locations) {
+            blame.appendf(FMT_STRING("{} -> {}:{}\n"),
+                          pair.first,
+                          pair.second.sl_source,
+                          pair.second.sl_line_number);
+        }
+
+        auto um = console::user_message::raw(blame.rtrim());
+
+        return {std::move(um)};
+    }
+
+    static perform_result_t file_options_action(const subcmd_config_t& sc)
+    {
+        auto& safe_options_hier
+            = injector::get<lnav::safe_file_options_hier&>();
+
+        if (sc.sc_path.empty()) {
+            auto um = lnav::console::user_message::error(
+                "Expecting a file path to check for options");
+
+            return {std::move(um)};
+        }
+
+        safe::ReadAccess<lnav::safe_file_options_hier> options_hier(
+            safe_options_hier);
+
+        auto realpath_res = lnav::filesystem::realpath(sc.sc_path);
+        if (realpath_res.isErr()) {
+            auto um = lnav::console::user_message::error(
+                          attr_line_t("Unable to get full path for file: ")
+                              .append(lnav::roles::file(sc.sc_path)))
+                          .with_reason(realpath_res.unwrapErr())
+                          .move();
+
+            return {std::move(um)};
+        }
+        auto full_path = realpath_res.unwrap();
+        auto file_opts = options_hier->match(full_path);
+        if (file_opts) {
+            auto content = attr_line_t()
+                               .append(file_opts->second.to_json_string()
+                                           .to_string_fragment())
+                               .move();
+            auto um = lnav::console::user_message::raw(content);
+            perform_result_t retval;
+
+            retval.emplace_back(um);
+
+            return retval;
+        }
+
+        auto um
+            = lnav::console::user_message::info(
+                  attr_line_t("no options found for file: ")
+                      .append(lnav::roles::file(full_path.string())))
+                  .with_help(
+                      attr_line_t("Use the ")
+                          .append(":set-file-timezone"_symbol)
+                          .append(
+                              " command to set the zone for messages in files "
+                              "that do not include a zone in the timestamp"))
+                  .move();
+
+        return {std::move(um)};
+    }
+
+    subcmd_config_t& set_action(action_t act)
+    {
+        if (!this->sc_action) {
+            this->sc_action = std::move(act);
+        }
+        return *this;
+    }
+};
+
 struct subcmd_format_t {
     using action_t = std::function<perform_result_t(const subcmd_format_t&)>;
 
@@ -89,31 +217,41 @@ struct subcmd_format_t {
         const
     {
         if (this->sf_name.empty()) {
-            auto um = console::user_message::error(
-                "expecting a format name to operate on");
-            um.with_note(
-                (log_format::get_root_formats()
-                 | lnav::itertools::map(&log_format::get_name)
-                 | lnav::itertools::sort_with(intern_string_t::case_lt)
-                 | lnav::itertools::map(&intern_string_t::to_string)
-                 | lnav::itertools::fold(symbol_reducer, attr_line_t{}))
-                    .add_header("the available formats are:"));
+            auto um
+                = console::user_message::error(
+                      "expecting a format name to operate on")
+                      .with_note(
+                          (log_format::get_root_formats()
+                           | lnav::itertools::map(&log_format::get_name)
+                           | lnav::itertools::sort_with(
+                               intern_string_t::case_lt)
+                           | lnav::itertools::map(&intern_string_t::to_string)
+                           | lnav::itertools::fold(symbol_reducer,
+                                                   attr_line_t{}))
+                              .add_header("the available formats are: ")
+                              .wrap_with(&DEFAULT_WRAPPING))
+                      .move();
 
             return Err(um);
         }
 
         auto lformat = log_format::find_root_format(this->sf_name.c_str());
         if (lformat == nullptr) {
-            auto um = console::user_message::error(
-                attr_line_t("unknown format: ")
-                    .append(lnav::roles::symbol(this->sf_name)));
-            um.with_note(
-                (log_format::get_root_formats()
-                 | lnav::itertools::map(&log_format::get_name)
-                 | lnav::itertools::similar_to(this->sf_name)
-                 | lnav::itertools::map(&intern_string_t::to_string)
-                 | lnav::itertools::fold(symbol_reducer, attr_line_t{}))
-                    .add_header("did you mean one of the following?"));
+            auto um
+                = console::user_message::error(
+                      attr_line_t("unknown format: ")
+                          .append(lnav::roles::symbol(this->sf_name)))
+                      .with_note(
+                          (log_format::get_root_formats()
+                           | lnav::itertools::map(&log_format::get_name)
+                           | lnav::itertools::similar_to(this->sf_name)
+                           | lnav::itertools::map(&intern_string_t::to_string)
+                           | lnav::itertools::fold(symbol_reducer,
+                                                   attr_line_t{}))
+                              .add_header(
+                                  "did you mean one of the following?\n")
+                              .wrap_with(&DEFAULT_WRAPPING))
+                      .move();
 
             return Err(um);
         }
@@ -146,14 +284,18 @@ struct subcmd_format_t {
         auto* ext_lformat = TRY(this->validate_external_format());
 
         if (this->sf_regex_name.empty()) {
-            auto um = console::user_message::error(
-                "expecting a regex name to operate on");
-            um.with_note(
-                ext_lformat->elf_pattern_order
-                | lnav::itertools::map(&external_log_format::pattern::p_name)
-                | lnav::itertools::map(&intern_string_t::to_string)
-                | lnav::itertools::fold(
-                    symbol_reducer, attr_line_t{"the available regexes are:"}));
+            auto um
+                = console::user_message::error(
+                      "expecting a regex name to operate on")
+                      .with_note(
+                          ext_lformat->elf_pattern_order
+                          | lnav::itertools::map(
+                              &external_log_format::pattern::p_name)
+                          | lnav::itertools::map(&intern_string_t::to_string)
+                          | lnav::itertools::fold(
+                              symbol_reducer,
+                              attr_line_t{"the available regexes are: "}))
+                      .move();
 
             return Err(um);
         }
@@ -164,16 +306,19 @@ struct subcmd_format_t {
             }
         }
 
-        auto um = console::user_message::error(
-            attr_line_t("unknown regex: ")
-                .append(lnav::roles::symbol(this->sf_regex_name)));
-        um.with_note(
-            (ext_lformat->elf_pattern_order
-             | lnav::itertools::map(&external_log_format::pattern::p_name)
-             | lnav::itertools::map(&intern_string_t::to_string)
-             | lnav::itertools::similar_to(this->sf_regex_name)
-             | lnav::itertools::fold(symbol_reducer, attr_line_t{}))
-                .add_header("did you mean one of the following?"));
+        auto um
+            = console::user_message::error(
+                  attr_line_t("unknown regex: ")
+                      .append(lnav::roles::symbol(this->sf_regex_name)))
+                  .with_note(
+                      (ext_lformat->elf_pattern_order
+                       | lnav::itertools::map(
+                           &external_log_format::pattern::p_name)
+                       | lnav::itertools::map(&intern_string_t::to_string)
+                       | lnav::itertools::similar_to(this->sf_regex_name)
+                       | lnav::itertools::fold(symbol_reducer, attr_line_t{}))
+                          .add_header("did you mean one of the following?\n"))
+                  .move();
 
         return Err(um);
     }
@@ -201,21 +346,23 @@ struct subcmd_format_t {
                       ", ");
         }
 
-        auto um = console::user_message::error(
-            attr_line_t("expecting an operation to perform on the ")
-                .append(lnav::roles::symbol(sf.sf_name))
-                .append(" format"));
-        um.with_note(attr_line_t()
-                         .append(lnav::roles::symbol(sf.sf_name))
-                         .append(": ")
-                         .append(lformat->lf_description)
-                         .append(ext_details));
-        um.with_help(
-            sf.sf_format_app->get_subcommands({})
-            | lnav::itertools::fold(
-                subcmd_reducer, attr_line_t{"the available operations are:"}));
+        auto um
+            = console::user_message::error(
+                  attr_line_t("expecting an operation to perform on the ")
+                      .append(lnav::roles::symbol(sf.sf_name))
+                      .append(" format"))
+                  .with_note(attr_line_t()
+                                 .append(lnav::roles::symbol(sf.sf_name))
+                                 .append(": ")
+                                 .append(lformat->lf_description)
+                                 .append(ext_details))
+                  .with_help(sf.sf_format_app->get_subcommands({})
+                             | lnav::itertools::fold(
+                                 subcmd_reducer,
+                                 attr_line_t{"the available operations are:"}))
+                  .move();
 
-        return {um};
+        return {std::move(um)};
     }
 
     static perform_result_t default_regex_action(const subcmd_format_t& sf)
@@ -227,15 +374,17 @@ struct subcmd_format_t {
         }
 
         auto um = console::user_message::error(
-            attr_line_t("expecting an operation to perform on the ")
-                .append(lnav::roles::symbol(sf.sf_regex_name))
-                .append(" regular expression"));
+                      attr_line_t("expecting an operation to perform on the ")
+                          .append(lnav::roles::symbol(sf.sf_regex_name))
+                          .append(" regular expression"))
+                      .with_help(
+                          attr_line_t{"the available subcommands are:"}.append(
+                              sf.sf_regex_app->get_subcommands({})
+                              | lnav::itertools::fold(subcmd_reducer,
+                                                      attr_line_t{})))
+                      .move();
 
-        um.with_help(attr_line_t{"the available subcommands are:"}.append(
-            sf.sf_regex_app->get_subcommands({})
-            | lnav::itertools::fold(subcmd_reducer, attr_line_t{})));
-
-        return {um};
+        return {std::move(um)};
     }
 
     static perform_result_t get_action(const subcmd_format_t& sf)
@@ -254,7 +403,7 @@ struct subcmd_format_t {
                 .append(": ")
                 .append(on_blank(format->lf_description, "<no description>")));
 
-        return {um};
+        return {std::move(um)};
     }
 
     static perform_result_t source_action(const subcmd_format_t& sf)
@@ -277,7 +426,7 @@ struct subcmd_format_t {
         auto um = console::user_message::raw(
             format->elf_format_source_order[0].string());
 
-        return {um};
+        return {std::move(um)};
     }
 
     static perform_result_t sources_action(const subcmd_format_t& sf)
@@ -302,7 +451,7 @@ struct subcmd_format_t {
                                VC_ROLE.value(role_t::VCR_TEXT),
                                "\n"));
 
-        return {um};
+        return {std::move(um)};
     }
 
     static perform_result_t regex101_pull_action(const subcmd_format_t& sf)
@@ -433,7 +582,7 @@ struct subcmd_format_t {
                                             .string())));
                         }
 
-                        return {um};
+                        return {std::move(um)};
                     });
             });
     }
@@ -466,7 +615,7 @@ struct subcmd_format_t {
             sf.sf_regex101_app->get_subcommands({})
             | lnav::itertools::fold(subcmd_reducer, attr_line_t{})));
 
-        return {um};
+        return {std::move(um)};
     }
 
     static perform_result_t regex101_push_action(const subcmd_format_t& sf)
@@ -548,7 +697,7 @@ struct subcmd_format_t {
                         auto ppath = regex101::patch_path(validate_res.unwrap(),
                                                           en.re_permalink);
 
-                        if (ghc::filesystem::exists(ppath)) {
+                        if (std::filesystem::exists(ppath)) {
                             return {
                                 console::user_message::error(
                                     attr_line_t("cannot delete regex101 entry "
@@ -622,6 +771,288 @@ struct subcmd_format_t {
     }
 };
 
+struct subcmd_piper_t {
+    using action_t = std::function<perform_result_t(const subcmd_piper_t&)>;
+
+    CLI::App* sp_app{nullptr};
+    action_t sp_action;
+
+    subcmd_piper_t& set_action(action_t act)
+    {
+        if (!this->sp_action) {
+            this->sp_action = std::move(act);
+        }
+        return *this;
+    }
+
+    static perform_result_t default_action(const subcmd_piper_t& sp)
+    {
+        auto um
+            = console::user_message::error(
+                  "expecting an operation related to piper storage")
+                  .with_help(sp.sp_app->get_subcommands({})
+                             | lnav::itertools::fold(
+                                 subcmd_reducer,
+                                 attr_line_t{"the available operations are:"}))
+                  .move();
+
+        return {std::move(um)};
+    }
+
+    static perform_result_t list_action(const subcmd_piper_t&)
+    {
+        static const intern_string_t SRC = intern_string::lookup("piper");
+        static const auto DOT_HEADER = std::filesystem::path(".header");
+
+        struct item {
+            lnav::piper::header i_header;
+            std::string i_url;
+            file_size_t i_total_size{0};
+        };
+
+        file_size_t grand_total{0};
+        std::vector<item> items;
+        std::error_code ec;
+
+        for (const auto& instance_dir : std::filesystem::directory_iterator(
+                 lnav::piper::storage_path(), ec))
+        {
+            if (!instance_dir.is_directory()) {
+                log_warning("piper directory entry is not a directory: %s",
+                            instance_dir.path().c_str());
+                continue;
+            }
+
+            std::optional<lnav::piper::header> hdr_opt;
+            auto url = fmt::format(FMT_STRING("piper://{}"),
+                                   instance_dir.path().filename().string());
+            file_size_t total_size{0};
+            auto hdr_path = instance_dir / DOT_HEADER;
+            if (std::filesystem::exists(hdr_path)) {
+                auto hdr_read_res = lnav::filesystem::read_file(hdr_path);
+                if (hdr_read_res.isOk()) {
+                    auto parse_res
+                        = lnav::piper::header_handlers.parser_for(SRC).of(
+                            hdr_read_res.unwrap());
+                    if (parse_res.isOk()) {
+                        hdr_opt = parse_res.unwrap();
+                    } else {
+                        log_error("failed to parse header: %s -- %s",
+                                  hdr_path.c_str(),
+                                  parse_res.unwrapErr()[0]
+                                      .to_attr_line()
+                                      .get_string()
+                                      .c_str());
+                    }
+                } else {
+                    log_error("failed to read header file: %s -- %s",
+                              hdr_path.c_str(),
+                              hdr_read_res.unwrapErr().c_str());
+                }
+            }
+
+            for (const auto& entry :
+                 std::filesystem::directory_iterator(instance_dir.path()))
+            {
+                if (entry.path().filename() == DOT_HEADER) {
+                    continue;
+                }
+
+                total_size += entry.file_size();
+                char buffer[lnav::piper::HEADER_SIZE];
+
+                auto entry_open_res
+                    = lnav::filesystem::open_file(entry.path(), O_RDONLY);
+                if (entry_open_res.isErr()) {
+                    log_warning("unable to open piper file: %s -- %s",
+                                entry.path().c_str(),
+                                entry_open_res.unwrapErr().c_str());
+                    continue;
+                }
+
+                auto entry_fd = entry_open_res.unwrap();
+                if (read(entry_fd, buffer, sizeof(buffer)) != sizeof(buffer)) {
+                    log_warning("piper file is too small: %s",
+                                entry.path().c_str());
+                    continue;
+                }
+                auto hdr_bits_opt = lnav::piper::read_header(entry_fd, buffer);
+                if (!hdr_bits_opt) {
+                    log_warning("could not read piper header: %s",
+                                entry.path().c_str());
+                    continue;
+                }
+
+                auto hdr_buf = std::move(hdr_bits_opt.value());
+
+                total_size -= hdr_buf.size();
+                auto hdr_sf
+                    = string_fragment::from_bytes(hdr_buf.in(), hdr_buf.size());
+                auto hdr_parse_res
+                    = lnav::piper::header_handlers.parser_for(SRC).of(hdr_sf);
+                if (hdr_parse_res.isErr()) {
+                    log_error("failed to parse piper header: %s",
+                              hdr_parse_res.unwrapErr()[0]
+                                  .to_attr_line()
+                                  .get_string()
+                                  .c_str());
+                    continue;
+                }
+
+                auto hdr = hdr_parse_res.unwrap();
+
+                if (!hdr_opt || hdr < hdr_opt.value()) {
+                    hdr_opt = hdr;
+                }
+            }
+
+            if (hdr_opt) {
+                items.emplace_back(item{hdr_opt.value(), url, total_size});
+            }
+
+            grand_total += total_size;
+        }
+
+        if (ec && ec.value() != ENOENT) {
+            auto um = lnav::console::user_message::error(
+                          attr_line_t("unable to access piper directory: ")
+                              .append(lnav::roles::file(
+                                  lnav::piper::storage_path().string())))
+                          .with_reason(ec.message())
+                          .move();
+            return {std::move(um)};
+        }
+
+        if (items.empty()) {
+            if (verbosity != verbosity_t::quiet) {
+                auto um
+                    = lnav::console::user_message::info(
+                          attr_line_t("no piper captures were found in:\n\t")
+                              .append(lnav::roles::file(
+                                  lnav::piper::storage_path().string())))
+                          .with_help(
+                              attr_line_t("You can create a capture by "
+                                          "piping data into ")
+                                  .append(lnav::roles::file("lnav"))
+                                  .append(" or using the ")
+                                  .append_quoted(lnav::roles::symbol(":sh"))
+                                  .append(" command"))
+                          .move();
+                return {std::move(um)};
+            }
+
+            return {};
+        }
+
+        auto txt
+            = items
+            | lnav::itertools::sort_with([](const item& lhs, const item& rhs) {
+                  if (lhs.i_header < rhs.i_header) {
+                      return true;
+                  }
+
+                  if (rhs.i_header < lhs.i_header) {
+                      return false;
+                  }
+
+                  return lhs.i_url < rhs.i_url;
+              })
+            | lnav::itertools::map([](const item& it) {
+                  auto ago = humanize::time::point::from_tv(it.i_header.h_ctime)
+                                 .as_time_ago();
+                  auto retval = attr_line_t()
+                                    .append(lnav::roles::list_glyph(
+                                        fmt::format(FMT_STRING("{:>18}"), ago)))
+                                    .append("  ")
+                                    .append(lnav::roles::file(it.i_url))
+                                    .append(" ")
+                                    .append(lnav::roles::number(fmt::format(
+                                        FMT_STRING("{:>8}"),
+                                        humanize::file_size(
+                                            it.i_total_size,
+                                            humanize::alignment::columnar))))
+                                    .append(" ")
+                                    .append_quoted(lnav::roles::comment(
+                                        it.i_header.h_name))
+                                    .append("\n");
+                  if (verbosity == verbosity_t::verbose) {
+                      auto env_al
+                          = it.i_header.h_env
+                          | lnav::itertools::map([](const auto& pair) {
+                                return attr_line_t()
+                                    .append(lnav::roles::identifier(pair.first))
+                                    .append("=")
+                                    .append(pair.second)
+                                    .append("\n");
+                            })
+                          | lnav::itertools::fold(
+                                [](const auto& elem, auto& accum) {
+                                    if (!accum.empty()) {
+                                        accum.append(28, ' ');
+                                    }
+                                    return accum.append(elem);
+                                },
+                                attr_line_t());
+
+                      retval.append(23, ' ')
+                          .append("cwd: ")
+                          .append(lnav::roles::file(it.i_header.h_cwd))
+                          .append("\n")
+                          .append(23, ' ')
+                          .append("env: ")
+                          .append(env_al);
+                  }
+                  return retval;
+              })
+            | lnav::itertools::fold(
+                  [](const auto& elem, auto& accum) {
+                      return accum.append(elem);
+                  },
+                  attr_line_t{});
+        txt.rtrim();
+
+        perform_result_t retval;
+        if (verbosity != verbosity_t::quiet) {
+            auto extra_um
+                = lnav::console::user_message::info(
+                      attr_line_t(
+                          "the following piper captures were found in:\n\t")
+                          .append(lnav::roles::file(
+                              lnav::piper::storage_path().string())))
+                      .with_note(
+                          attr_line_t("The captures currently consume ")
+                              .append(lnav::roles::number(humanize::file_size(
+                                  grand_total, humanize::alignment::none)))
+                              .append(" of disk space.  File sizes include "
+                                      "associated metadata."))
+                      .with_help(
+                          "You can reopen a capture by passing the piper URL "
+                          "to lnav")
+                      .move();
+            retval.emplace_back(extra_um);
+        }
+        retval.emplace_back(lnav::console::user_message::raw(txt));
+
+        return retval;
+    }
+
+    static perform_result_t clean_action(const subcmd_piper_t&)
+    {
+        std::error_code ec;
+
+        std::filesystem::remove_all(lnav::piper::storage_path(), ec);
+        if (ec) {
+            return {
+                lnav::console::user_message::error(
+                    "unable to remove piper storage directory")
+                    .with_reason(ec.message()),
+            };
+        }
+
+        return {};
+    }
+};
+
 struct subcmd_regex101_t {
     using action_t = std::function<perform_result_t(const subcmd_regex101_t&)>;
 
@@ -641,14 +1072,17 @@ struct subcmd_regex101_t {
 
     static perform_result_t default_action(const subcmd_regex101_t& sr)
     {
-        auto um = console::user_message::error(
-            "expecting an operation related to the regex101.com integration");
-        um.with_help(
-            sr.sr_app->get_subcommands({})
-            | lnav::itertools::fold(
-                subcmd_reducer, attr_line_t{"the available operations are:"}));
+        auto um
+            = console::user_message::error(
+                  "expecting an operation related to the regex101.com "
+                  "integration")
+                  .with_help(sr.sr_app->get_subcommands({})
+                             | lnav::itertools::fold(
+                                 subcmd_reducer,
+                                 attr_line_t{"the available operations are:"}))
+                  .move();
 
-        return {um};
+        return {std::move(um)};
     }
 
     static perform_result_t list_action(const subcmd_regex101_t&)
@@ -663,24 +1097,24 @@ struct subcmd_regex101_t {
             };
         }
 
-        auto entries
-            = get_res.unwrap() | lnav::itertools::map([](const auto& elem) {
+        auto entries = get_res.unwrap()
+            | lnav::itertools::map([](const auto& elem) {
                            return fmt::format(
                                FMT_STRING("   format {} regex {} regex101\n"),
                                elem.re_format_name,
                                elem.re_regex_name);
                        })
             | lnav::itertools::fold(
-                  [](const auto& elem, auto& accum) {
-                      return accum.append(elem);
-                  },
-                  attr_line_t{});
+                           [](const auto& elem, auto& accum) {
+                               return accum.append(elem);
+                           },
+                           attr_line_t{});
 
         auto um = console::user_message::ok(
             entries.add_header("the following regex101 entries were found:\n")
                 .with_default("no regex101 entries found"));
 
-        return {um};
+        return {std::move(um)};
     }
 
     static perform_result_t import_action(const subcmd_regex101_t& sr)
@@ -711,8 +1145,102 @@ struct subcmd_regex101_t {
     }
 };
 
-using operations_v
-    = mapbox::util::variant<no_subcmd_t, subcmd_format_t, subcmd_regex101_t>;
+struct subcmd_crash_t {
+    using action_t = std::function<perform_result_t(const subcmd_crash_t&)>;
+
+    CLI::App* sc_app{nullptr};
+    action_t sc_action;
+
+    subcmd_crash_t& set_action(action_t act)
+    {
+        if (!this->sc_action) {
+            this->sc_action = std::move(act);
+        }
+        return *this;
+    }
+
+    static perform_result_t default_action(const subcmd_crash_t& sc)
+    {
+        auto um
+            = console::user_message::error(
+                  "expecting an operation related to crash logs")
+                  .with_help(sc.sc_app->get_subcommands({})
+                             | lnav::itertools::fold(
+                                 subcmd_reducer,
+                                 attr_line_t{"the available operations are:"}))
+                  .move();
+
+        return {std::move(um)};
+    }
+
+    static perform_result_t upload_action(const subcmd_crash_t&)
+    {
+        static constexpr char SPINNER_CHARS[] = "-\\|/";
+        constexpr size_t SPINNER_SIZE = sizeof(SPINNER_CHARS) - 1;
+
+        static_root_mem<glob_t, globfree> gl;
+        const auto path = lnav::paths::dotlnav() / "crash" / "crash-*.log";
+        perform_result_t retval;
+
+        auto glob_rc = glob(path.c_str(), 0, nullptr, gl.inout());
+        if (glob_rc == GLOB_NOMATCH) {
+            auto um = console::user_message::info("no crash logs to upload");
+            return {std::move(um)};
+        }
+        if (glob_rc != 0) {
+            auto um = console::user_message::error("unable to find crash logs");
+            return {std::move(um)};
+        }
+
+        for (size_t lpc = 0; lpc < gl->gl_pathc; lpc++) {
+            auto crash_file = std::filesystem::path(gl->gl_pathv[lpc]);
+            int spinner_index = 0;
+
+            log_info("uploading crash log: %s", crash_file.c_str());
+            printf("~");
+            fflush(stdout);
+            auto upload_res = crashd::client::upload(
+                crash_file,
+                [&spinner_index](double dltotal,
+                                 double dlnow,
+                                 double ultotal,
+                                 double ulnow) {
+                    printf("\b%c", SPINNER_CHARS[spinner_index % SPINNER_SIZE]);
+                    spinner_index += 1;
+                    fflush(stdout);
+                    return crashd::client::progress_result_t::ok;
+                });
+            if (spinner_index > 0) {
+                printf("\b");
+            }
+            printf(".");
+            fflush(stdout);
+            if (upload_res.isErr()) {
+                retval.push_back(upload_res.unwrapErr());
+            } else {
+                std::error_code ec;
+
+                std::filesystem::remove(crash_file, ec);
+            }
+        }
+
+        printf("\n");
+        auto um = console::user_message::ok(
+            attr_line_t("uploaded ")
+                .append(lnav::roles::number(fmt::to_string(gl->gl_pathc)))
+                .append(" crash logs, thank you!"));
+        retval.push_back(um);
+
+        return retval;
+    }
+};
+
+using operations_v = mapbox::util::variant<no_subcmd_t,
+                                           subcmd_config_t,
+                                           subcmd_format_t,
+                                           subcmd_piper_t,
+                                           subcmd_regex101_t,
+                                           subcmd_crash_t>;
 
 class operations {
 public:
@@ -730,8 +1258,42 @@ describe_cli(CLI::App& app, int argc, char* argv[])
 
     app.add_flag("-m", "Switch to the management CLI mode.");
 
+    subcmd_config_t config_args;
     subcmd_format_t format_args;
+    subcmd_piper_t piper_args;
     subcmd_regex101_t regex101_args;
+    subcmd_crash_t crash_args;
+
+    {
+        auto* subcmd_config
+            = app.add_subcommand("config",
+                                 "perform operations on the lnav configuration")
+                  ->callback([&]() {
+                      config_args.set_action(subcmd_config_t::default_action);
+                      retval->o_ops = config_args;
+                  });
+        config_args.sc_config_app = subcmd_config;
+
+        subcmd_config->add_subcommand("get", "print the current configuration")
+            ->callback(
+                [&]() { config_args.set_action(subcmd_config_t::get_action); });
+
+        subcmd_config
+            ->add_subcommand("blame",
+                             "print the configuration options and their source")
+            ->callback([&]() {
+                config_args.set_action(subcmd_config_t::blame_action);
+            });
+
+        auto* sub_file_options = subcmd_config->add_subcommand(
+            "file-options", "print the options applied to specific files");
+
+        sub_file_options->add_option(
+            "path", config_args.sc_path, "the path to the file");
+        sub_file_options->callback([&]() {
+            config_args.set_action(subcmd_config_t::file_options_action);
+        });
+    }
 
     {
         auto* subcmd_format
@@ -836,6 +1398,25 @@ describe_cli(CLI::App& app, int argc, char* argv[])
         }
     }
 
+    {
+        auto* subcmd_piper
+            = app.add_subcommand("piper", "perform operations on piper storage")
+                  ->callback([&]() {
+                      piper_args.set_action(subcmd_piper_t::default_action);
+                      retval->o_ops = piper_args;
+                  });
+        piper_args.sp_app = subcmd_piper;
+
+        subcmd_piper
+            ->add_subcommand("list", "print the available piper captures")
+            ->callback(
+                [&]() { piper_args.set_action(subcmd_piper_t::list_action); });
+
+        subcmd_piper->add_subcommand("clean", "remove all piper captures")
+            ->callback(
+                [&]() { piper_args.set_action(subcmd_piper_t::clean_action); });
+    }
+
     {
         auto* subcmd_regex101
             = app.add_subcommand("regex101",
@@ -883,6 +1464,22 @@ describe_cli(CLI::App& app, int argc, char* argv[])
         }
     }
 
+    {
+        auto* subcmd_crash
+            = app.add_subcommand("crash", "manage crash logs")->callback([&]() {
+                  crash_args.set_action(subcmd_crash_t::default_action);
+                  retval->o_ops = crash_args;
+              });
+        crash_args.sc_app = subcmd_crash;
+
+        {
+            subcmd_crash->add_subcommand("upload", "upload crash logs")
+                ->callback([&]() {
+                    crash_args.set_action(subcmd_crash_t::upload_action);
+                });
+        }
+    }
+
     app.parse(argc, argv);
 
     return retval;
@@ -894,16 +1491,21 @@ perform(std::shared_ptr<operations> opts)
     return opts->o_ops.match(
         [](const no_subcmd_t& ns) -> perform_result_t {
             auto um = console::user_message::error(
-                attr_line_t("expecting an operation to perform"));
-            um.with_help(ns.ns_root_app->get_subcommands({})
-                         | lnav::itertools::fold(
-                             subcmd_reducer,
-                             attr_line_t{"the available operations are:"}));
+                          attr_line_t("expecting an operation to perform"))
+                          .with_help(
+                              ns.ns_root_app->get_subcommands({})
+                              | lnav::itertools::fold(
+                                  subcmd_reducer,
+                                  attr_line_t{"the available operations are:"}))
+                          .move();
 
-            return {um};
+            return {std::move(um)};
         },
+        [](const subcmd_config_t& sc) { return sc.sc_action(sc); },
         [](const subcmd_format_t& sf) { return sf.sf_action(sf); },
-        [](const subcmd_regex101_t& sr) { return sr.sr_action(sr); });
+        [](const subcmd_piper_t& sp) { return sp.sp_action(sp); },
+        [](const subcmd_regex101_t& sr) { return sr.sr_action(sr); },
+        [](const subcmd_crash_t& sc) { return sc.sc_action(sc); });
 }
 
 }  // namespace management
