@@ -41,10 +41,6 @@
 #include "sql_util.hh"
 #include "view_curses.hh"
 
-static void readline_sqlite_highlighter_int(attr_line_t& al,
-                                            int x,
-                                            line_range sub);
-
 static bool
 is_bracket(const std::string& str, int index, bool is_lit)
 {
@@ -61,7 +57,7 @@ static void
 find_matching_bracket(
     attr_line_t& al, int x, line_range sub, char left, char right)
 {
-    bool is_lit = (left == 'Q');
+    const auto is_lit = left == 'Q';
     attr_line_builder alb(al);
     const auto& line = al.get_string();
     int depth = 0;
@@ -104,7 +100,7 @@ find_matching_bracket(
         }
     }
 
-    nonstd::optional<int> first_left;
+    std::optional<int> first_left;
 
     depth = 0;
 
@@ -136,14 +132,16 @@ find_matching_bracket(
 }
 
 void
-readline_regex_highlighter(attr_line_t& al, int x)
+readline_regex_highlighter(attr_line_t& al, std::optional<int> x)
 {
     lnav::snippets::regex_highlighter(
         al, x, line_range{1, (int) al.get_string().size()});
 }
 
 void
-readline_command_highlighter_int(attr_line_t& al, int x, line_range sub)
+readline_command_highlighter_int(attr_line_t& al,
+                                 std::optional<int> x,
+                                 line_range sub)
 {
     static const auto RE_PREFIXES = lnav::pcre2pp::code::from_const(
         R"(^:(filter-in|filter-out|delete-filter|enable-filter|disable-filter|highlight|clear-highlight|create-search-table\s+[^\s]+\s+))");
@@ -190,11 +188,11 @@ readline_command_highlighter_int(attr_line_t& al, int x, line_range sub)
                     .then([&](const auto& rgb_fg) {
                         auto color
                             = view_colors::singleton().match_color(rgb_fg);
-                        alb.template overlay_attr(to_line_range(md[0].value()),
-                                                  VC_STYLE.value(text_attrs{
-                                                      A_BOLD,
-                                                      color,
-                                                  }));
+                        alb.overlay_attr(to_line_range(md[0].value()),
+                                         VC_STYLE.value(text_attrs{
+                                             A_BOLD,
+                                             color,
+                                         }));
                     });
             });
     }
@@ -213,7 +211,9 @@ readline_command_highlighter_int(attr_line_t& al, int x, line_range sub)
                 (int) start, (int) last
             };
 
-            if (lr.length() > 0 && !lr.contains(x) && !lr.contains(x - 1)) {
+            if (x && lr.length() > 0 && !lr.contains(x.value())
+                && !lr.contains(x.value() - 1))
+            {
                 std::string value(lr.substr(line), lr.sublen(line));
 
                 if ((command == ":tag" || command == ":untag"
@@ -231,20 +231,21 @@ readline_command_highlighter_int(attr_line_t& al, int x, line_range sub)
 }
 
 void
-readline_command_highlighter(attr_line_t& al, int x)
+readline_command_highlighter(attr_line_t& al, std::optional<int> x)
 {
     readline_command_highlighter_int(
         al, x, line_range{0, (int) al.get_string().length()});
 }
 
-static void
-readline_sqlite_highlighter_int(attr_line_t& al, int x, line_range sub)
+void
+readline_sqlite_highlighter_int(attr_line_t& al,
+                                std::optional<int> x,
+                                line_range sub)
 {
     static const char* brackets[] = {
         "[]",
         "()",
-
-        nullptr,
+        "{}",
     };
 
     attr_line_builder alb(al);
@@ -260,18 +261,26 @@ readline_sqlite_highlighter_int(attr_line_t& al, int x, line_range sub)
             sub.lr_start + attr.sa_range.lr_end,
         };
         if (attr.sa_type == &SQL_COMMAND_ATTR
-            || attr.sa_type == &SQL_KEYWORD_ATTR)
+            || attr.sa_type == &SQL_KEYWORD_ATTR
+            || attr.sa_type == &lnav::sql::PRQL_KEYWORD_ATTR
+            || attr.sa_type == &lnav::sql::PRQL_TRANSFORM_ATTR)
         {
             alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_KEYWORD));
-        } else if (attr.sa_type == &SQL_IDENTIFIER_ATTR) {
-            if (!attr.sa_range.contains(x) && attr.sa_range.lr_end != x) {
+        } else if (attr.sa_type == &SQL_IDENTIFIER_ATTR
+                   || attr.sa_type == &lnav::sql::PRQL_IDENTIFIER_ATTR)
+        {
+            if (x && !attr.sa_range.contains(x.value())
+                && attr.sa_range.lr_end != x.value())
+            {
                 alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_IDENTIFIER));
             }
         } else if (attr.sa_type == &SQL_FUNCTION_ATTR) {
             alb.overlay_attr(
                 line_range{lr.lr_start, (int) line.find('(', lr.lr_start)},
                 VC_ROLE.value(role_t::VCR_SYMBOL));
-        } else if (attr.sa_type == &SQL_NUMBER_ATTR) {
+        } else if (attr.sa_type == &SQL_NUMBER_ATTR
+                   || attr.sa_type == &lnav::sql::PRQL_NUMBER_ATTR)
+        {
             alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_NUMBER));
         } else if (attr.sa_type == &SQL_STRING_ATTR) {
             if (lr.length() > 1 && al.al_string[lr.lr_end - 1] == '\'') {
@@ -282,87 +291,110 @@ readline_sqlite_highlighter_int(attr_line_t& al, int x, line_range sub)
                 alb.overlay_attr_for_char(lr.lr_start,
                                           VC_ROLE.value(role_t::VCR_ERROR));
             }
-        } else if (attr.sa_type == &SQL_OPERATOR_ATTR) {
+        } else if (attr.sa_type == &lnav::sql::PRQL_STRING_ATTR) {
+            alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_STRING));
+        } else if (attr.sa_type == &SQL_OPERATOR_ATTR
+                   || attr.sa_type == &lnav::sql::PRQL_OPERATOR_ATTR)
+        {
             alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_SYMBOL));
-        } else if (attr.sa_type == &SQL_COMMENT_ATTR) {
+        } else if (attr.sa_type == &SQL_COMMENT_ATTR
+                   || attr.sa_type == &lnav::sql::PRQL_COMMENT_ATTR)
+        {
             alb.overlay_attr(lr, VC_ROLE.value(role_t::VCR_COMMENT));
         }
     }
 
-    for (int lpc = 0; brackets[lpc]; lpc++) {
-        find_matching_bracket(al, x, sub, brackets[lpc][0], brackets[lpc][1]);
+    for (const auto& bracket : brackets) {
+        find_matching_bracket(
+            al, x.value_or(al.length()), sub, bracket[0], bracket[1]);
     }
 }
 
 void
-readline_sqlite_highlighter(attr_line_t& al, int x)
+readline_sqlite_highlighter(attr_line_t& al, std::optional<int> x)
 {
     readline_sqlite_highlighter_int(
         al, x, line_range{0, (int) al.get_string().length()});
 }
 
 void
-readline_shlex_highlighter_int(attr_line_t& al, int x, line_range sub)
+readline_shlex_highlighter_int(attr_line_t& al,
+                               std::optional<int> x,
+                               line_range sub)
 {
     attr_line_builder alb(al);
     const auto& str = al.get_string();
-    string_fragment cap;
-    shlex_token_t token;
-    nonstd::optional<int> quote_start;
+    std::optional<int> quote_start;
     shlex lexer(string_fragment{al.al_string.data(), sub.lr_start, sub.lr_end});
+    bool done = false;
 
-    while (lexer.tokenize(cap, token)) {
-        switch (token) {
-            case shlex_token_t::ST_ERROR:
-                alb.overlay_attr(line_range(sub.lr_start + cap.sf_begin,
-                                            sub.lr_start + cap.sf_end),
-                                 VC_STYLE.value(text_attrs{A_REVERSE}));
-                alb.overlay_attr(line_range(sub.lr_start + cap.sf_begin,
-                                            sub.lr_start + cap.sf_end),
-                                 VC_ROLE.value(role_t::VCR_ERROR));
+    while (!done) {
+        auto tokenize_res = lexer.tokenize();
+        if (tokenize_res.isErr()) {
+            auto te = tokenize_res.unwrapErr();
+
+            alb.overlay_attr(line_range(sub.lr_start + te.te_source.sf_begin,
+                                        sub.lr_start + te.te_source.sf_end),
+                             VC_STYLE.value(text_attrs{A_REVERSE}));
+            alb.overlay_attr(line_range(sub.lr_start + te.te_source.sf_begin,
+                                        sub.lr_start + te.te_source.sf_end),
+                             VC_ROLE.value(role_t::VCR_ERROR));
+            return;
+        }
+
+        auto token = tokenize_res.unwrap();
+        switch (token.tr_token) {
+            case shlex_token_t::eof:
+                done = true;
                 break;
-            case shlex_token_t::ST_TILDE:
-            case shlex_token_t::ST_ESCAPE:
-                alb.overlay_attr(line_range(sub.lr_start + cap.sf_begin,
-                                            sub.lr_start + cap.sf_end),
-                                 VC_ROLE.value(role_t::VCR_SYMBOL));
-                break;
-            case shlex_token_t::ST_DOUBLE_QUOTE_START:
-            case shlex_token_t::ST_SINGLE_QUOTE_START:
-                quote_start = sub.lr_start + cap.sf_begin;
-                break;
-            case shlex_token_t::ST_DOUBLE_QUOTE_END:
-            case shlex_token_t::ST_SINGLE_QUOTE_END:
+            case shlex_token_t::tilde:
+            case shlex_token_t::escape:
                 alb.overlay_attr(
-                    line_range(quote_start.value(), sub.lr_start + cap.sf_end),
+                    line_range(sub.lr_start + token.tr_frag.sf_begin,
+                               sub.lr_start + token.tr_frag.sf_end),
+                    VC_ROLE.value(role_t::VCR_SYMBOL));
+                break;
+            case shlex_token_t::double_quote_start:
+            case shlex_token_t::single_quote_start:
+                quote_start = sub.lr_start + token.tr_frag.sf_begin;
+                break;
+            case shlex_token_t::double_quote_end:
+            case shlex_token_t::single_quote_end:
+                alb.overlay_attr(
+                    line_range(quote_start.value(),
+                               sub.lr_start + token.tr_frag.sf_end),
                     VC_ROLE.value(role_t::VCR_STRING));
-                quote_start = nonstd::nullopt;
+                quote_start = std::nullopt;
                 break;
-            case shlex_token_t::ST_VARIABLE_REF:
-            case shlex_token_t::ST_QUOTED_VARIABLE_REF: {
-                int extra = token == shlex_token_t::ST_VARIABLE_REF ? 0 : 1;
-                auto ident = str.substr(sub.lr_start + cap.sf_begin + 1 + extra,
-                                        cap.length() - 1 - extra * 2);
+            case shlex_token_t::variable_ref:
+            case shlex_token_t::quoted_variable_ref: {
+                int extra = token.tr_token == shlex_token_t::variable_ref ? 0
+                                                                          : 1;
+                auto ident = str.substr(
+                    sub.lr_start + token.tr_frag.sf_begin + 1 + extra,
+                    token.tr_frag.length() - 1 - extra * 2);
                 alb.overlay_attr(
-                    line_range(sub.lr_start + cap.sf_begin,
-                               sub.lr_start + cap.sf_begin + 1 + extra),
+                    line_range(
+                        sub.lr_start + token.tr_frag.sf_begin,
+                        sub.lr_start + token.tr_frag.sf_begin + 1 + extra),
                     VC_ROLE.value(role_t::VCR_SYMBOL));
                 alb.overlay_attr(
-                    line_range(sub.lr_start + cap.sf_begin + 1 + extra,
-                               sub.lr_start + cap.sf_end - extra),
-                    VC_ROLE.value(
-                        x == sub.lr_start + cap.sf_end
-                                || (cap.sf_begin <= x && x < cap.sf_end)
-                            ? role_t::VCR_SYMBOL
-                            : role_t::VCR_IDENTIFIER));
+                    line_range(
+                        sub.lr_start + token.tr_frag.sf_begin + 1 + extra,
+                        sub.lr_start + token.tr_frag.sf_end - extra),
+                    VC_ROLE.value(x == sub.lr_start + token.tr_frag.sf_end
+                                          || (token.tr_frag.sf_begin <= x
+                                              && x < token.tr_frag.sf_end)
+                                      ? role_t::VCR_SYMBOL
+                                      : role_t::VCR_IDENTIFIER));
                 if (extra) {
                     alb.overlay_attr_for_char(
-                        sub.lr_start + cap.sf_end - 1,
+                        sub.lr_start + token.tr_frag.sf_end - 1,
                         VC_ROLE.value(role_t::VCR_SYMBOL));
                 }
                 break;
             }
-            case shlex_token_t::ST_WHITESPACE:
+            case shlex_token_t::whitespace:
                 break;
         }
     }
@@ -374,14 +406,16 @@ readline_shlex_highlighter_int(attr_line_t& al, int x, line_range sub)
 }
 
 void
-readline_shlex_highlighter(attr_line_t& al, int x)
+readline_shlex_highlighter(attr_line_t& al, std::optional<int> x)
 {
     readline_shlex_highlighter_int(
         al, x, line_range{0, (int) al.al_string.length()});
 }
 
 static void
-readline_lnav_highlighter_int(attr_line_t& al, int x, line_range sub)
+readline_lnav_highlighter_int(attr_line_t& al,
+                              std::optional<int> x,
+                              line_range sub)
 {
     switch (al.al_string[sub.lr_start]) {
         case ':':
@@ -409,13 +443,13 @@ readline_lnav_highlighter_int(attr_line_t& al, int x, line_range sub)
 }
 
 void
-readline_lnav_highlighter(attr_line_t& al, int x)
+readline_lnav_highlighter(attr_line_t& al, std::optional<int> x)
 {
     static const auto COMMENT_RE = lnav::pcre2pp::code::from_const(R"(^\s*#)");
 
     attr_line_builder alb(al);
     size_t start = 0, lf_pos;
-    nonstd::optional<size_t> section_start;
+    std::optional<size_t> section_start;
 
     while ((lf_pos = al.get_string().find('\n', start)) != std::string::npos) {
         line_range line{(int) start, (int) lf_pos};
@@ -437,7 +471,7 @@ readline_lnav_highlighter(attr_line_t& al, int x)
                                                   (int) section_start.value(),
                                                   line.lr_start,
                                               });
-                section_start = nonstd::nullopt;
+                section_start = std::nullopt;
             }
             alb.overlay_attr(line_range{find_res->f_all.sf_begin, line.lr_end},
                              VC_ROLE.value(role_t::VCR_COMMENT));
