@@ -31,6 +31,7 @@
 #    include <alloca.h>
 #endif
 
+#include "base/lnav.console.into.hh"
 #include "base/lnav_log.hh"
 #include "column_namer.hh"
 #include "config.h"
@@ -39,10 +40,13 @@
 #include "scn/scn.h"
 #include "sql_help.hh"
 #include "sql_util.hh"
+#include "sqlitepp.hh"
 #include "vtab_module.hh"
 #include "yajlpp/yajlpp.hh"
 #include "yajlpp/yajlpp_def.hh"
 
+namespace {
+
 enum {
     RC_COL_MATCH_INDEX,
     RC_COL_INDEX,
@@ -238,9 +242,12 @@ rcFilter(sqlite3_vtab_cursor* pVtabCursor,
     auto pattern = from_sqlite<string_fragment>()(argc, argv, 1);
     auto compile_res = lnav::pcre2pp::code::from(pattern);
     if (compile_res.isErr()) {
-        pVtabCursor->pVtab->zErrMsg
-            = sqlite3_mprintf("Invalid regular expression: %s",
-                              compile_res.unwrapErr().get_message().c_str());
+        static const intern_string_t PATTERN_SRC
+            = intern_string::lookup("pattern");
+
+        set_vtable_errmsg(pVtabCursor->pVtab,
+                          lnav::console::to_user_message(
+                              PATTERN_SRC, compile_res.unwrapErr()));
         return SQLITE_ERROR;
     }
 
@@ -310,7 +317,7 @@ CREATE TABLE regexp_capture_into_json (
         size_t c_match_index{0};
         sqlite3_int64 c_rowid{0};
         std::string c_flags_string;
-        nonstd::optional<regexp_capture_flags> c_flags;
+        std::optional<regexp_capture_flags> c_flags;
 
         cursor(sqlite3_vtab* vt) : base({vt}) {}
 
@@ -473,7 +480,7 @@ rcjFilter(sqlite3_vtab_cursor* pVtabCursor,
         pCur->c_content.clear();
         pCur->c_pattern.reset();
         pCur->c_flags_string.clear();
-        pCur->c_flags = nonstd::nullopt;
+        pCur->c_flags = std::nullopt;
         return SQLITE_OK;
     }
 
@@ -486,14 +493,17 @@ rcjFilter(sqlite3_vtab_cursor* pVtabCursor,
     auto pattern = from_sqlite<string_fragment>()(argc, argv, 1);
     auto compile_res = lnav::pcre2pp::code::from(pattern);
     if (compile_res.isErr()) {
-        pVtabCursor->pVtab->zErrMsg
-            = sqlite3_mprintf("Invalid regular expression: %s",
-                              compile_res.unwrapErr().get_message().c_str());
+        static const intern_string_t PATTERN_SRC
+            = intern_string::lookup("pattern");
+
+        set_vtable_errmsg(pVtabCursor->pVtab,
+                          lnav::console::to_user_message(
+                              PATTERN_SRC, compile_res.unwrapErr()));
         return SQLITE_ERROR;
     }
 
     pCur->c_flags_string.clear();
-    pCur->c_flags = nonstd::nullopt;
+    pCur->c_flags = std::nullopt;
     if (argc == 3) {
         static const intern_string_t FLAGS_SRC = intern_string::lookup("flags");
         const auto flags_json = from_sqlite<string_fragment>()(argc, argv, 2);
@@ -504,12 +514,12 @@ rcjFilter(sqlite3_vtab_cursor* pVtabCursor,
                     flags_json);
 
             if (parse_res.isErr()) {
-                auto um = lnav::console::user_message::error(
-                              "unable to parse flags")
-                              .with_reason(parse_res.unwrapErr()[0]);
+                const auto um = lnav::console::user_message::error(
+                                    "unable to parse flags")
+                                    .with_reason(parse_res.unwrapErr()[0])
+                                    .move();
 
-                pVtabCursor->pVtab->zErrMsg = sqlite3_mprintf(
-                    "%s%s", sqlitepp::ERROR_PREFIX, lnav::to_json(um).c_str());
+                set_vtable_errmsg(pVtabCursor->pVtab, um);
                 return SQLITE_ERROR;
             }
 
@@ -542,6 +552,8 @@ rcjFilter(sqlite3_vtab_cursor* pVtabCursor,
     return SQLITE_OK;
 }
 
+}  // namespace
+
 int
 register_regexp_vtab(sqlite3* db)
 {
