@@ -31,6 +31,7 @@
 
 #include "relative_time.hh"
 
+#include "base/lnav_log.hh"
 #include "base/time_util.hh"
 #include "config.h"
 #include "pcrepp/pcre2pp.hh"
@@ -294,7 +295,8 @@ relative_time::from_str(string_fragment str)
 
         bool found = false;
         for (int lpc = 0; lpc < RTT__MAX && !found; lpc++) {
-            static thread_local auto md = lnav::pcre2pp::match_data::unitialized();
+            static thread_local auto md
+                = lnav::pcre2pp::match_data::unitialized();
 
             token_t token = (token_t) lpc;
             auto match_res = MATCHERS[lpc]
@@ -929,6 +931,7 @@ relative_time::adjust(const exttm& tm) const
         retval.et_tm.tm_min += this->rt_field[RTF_MINUTES].value;
     }
     if (this->rt_field[RTF_HOURS].is_set && this->is_absolute(RTF_HOURS)) {
+        retval.et_tm.tm_yday = -1;
         if (this->rt_next
             && this->rt_field[RTF_HOURS].value <= tm.et_tm.tm_hour)
         {
@@ -953,8 +956,10 @@ relative_time::adjust(const exttm& tm) const
         {
             retval.et_tm.tm_mon -= 1;
         }
+        retval.et_tm.tm_yday = -1;
         retval.et_tm.tm_mday = this->rt_field[RTF_DAYS].value;
     } else {
+        retval.et_tm.tm_yday = -1;
         retval.et_tm.tm_mday += this->rt_field[RTF_DAYS].value;
     }
     if (this->rt_field[RTF_MONTHS].is_set && this->is_absolute(RTF_MONTHS)) {
@@ -981,7 +986,7 @@ relative_time::adjust(const exttm& tm) const
     return retval;
 }
 
-nonstd::optional<exttm>
+std::optional<exttm>
 relative_time::window_start(const struct exttm& tm) const
 {
     auto retval = tm;
@@ -1007,7 +1012,7 @@ relative_time::window_start(const struct exttm& tm) const
 
     if (this->rt_field[RTF_YEARS].is_set) {
         if (this->rt_field[RTF_YEARS].value > tm.et_tm.tm_year) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
         retval.et_tm.tm_year = this->rt_field[RTF_YEARS].value;
         clear = true;
@@ -1015,21 +1020,25 @@ relative_time::window_start(const struct exttm& tm) const
 
     if (this->rt_field[RTF_MONTHS].is_set) {
         if (this->rt_field[RTF_MONTHS].value > tm.et_tm.tm_mon) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
+        retval.et_tm.tm_yday = -1;
         retval.et_tm.tm_mon = this->rt_field[RTF_MONTHS].value;
         clear = true;
     } else if (clear) {
+        retval.et_tm.tm_yday = -1;
         retval.et_tm.tm_mon = 0;
     }
 
     if (this->rt_field[RTF_DAYS].is_set) {
         if (this->rt_field[RTF_DAYS].value > tm.et_tm.tm_mday) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
+        retval.et_tm.tm_yday = -1;
         retval.et_tm.tm_mday = this->rt_field[RTF_DAYS].value;
         clear = true;
     } else if (clear) {
+        retval.et_tm.tm_yday = -1;
         retval.et_tm.tm_mday = 1;
     }
 
@@ -1038,14 +1047,14 @@ relative_time::window_start(const struct exttm& tm) const
             (token_t) (RTT_SUNDAY + tm.et_tm.tm_wday));
 
         if (iter == this->rt_included_days.end()) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
         clear = true;
     }
 
     if (this->rt_field[RTF_HOURS].is_set) {
         if (this->rt_field[RTF_HOURS].value > tm.et_tm.tm_hour) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
         retval.et_tm.tm_hour = this->rt_field[RTF_HOURS].value;
         clear = true;
@@ -1055,7 +1064,7 @@ relative_time::window_start(const struct exttm& tm) const
 
     if (this->rt_field[RTF_MINUTES].is_set) {
         if (this->rt_field[RTF_MINUTES].value > tm.et_tm.tm_min) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
         retval.et_tm.tm_min = this->rt_field[RTF_MINUTES].value;
         clear = true;
@@ -1065,7 +1074,7 @@ relative_time::window_start(const struct exttm& tm) const
 
     if (this->rt_field[RTF_SECONDS].is_set) {
         if (this->rt_field[RTF_SECONDS].value > tm.et_tm.tm_sec) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
         retval.et_tm.tm_sec = this->rt_field[RTF_SECONDS].value;
         clear = true;
@@ -1075,7 +1084,7 @@ relative_time::window_start(const struct exttm& tm) const
 
     if (this->rt_field[RTF_MICROSECONDS].is_set) {
         if (this->rt_field[RTF_MICROSECONDS].value > tm.et_nsec / 1000) {
-            return nonstd::nullopt;
+            return std::nullopt;
         }
         retval.et_nsec = this->rt_field[RTF_MICROSECONDS].value * 1000ULL;
         clear = true;
@@ -1090,7 +1099,7 @@ relative_time::window_start(const struct exttm& tm) const
                         .to_timeval();
 
     if (tv < start_time || end_time < tv) {
-        return nonstd::nullopt;
+        return std::nullopt;
     }
 
     return retval;
@@ -1112,10 +1121,11 @@ relative_time::to_microseconds() const
         } else {
             etm.et_tm.tm_mday = 1;
         }
+        etm.et_tm.tm_yday = -1;
         etm.et_tm.tm_min = this->rt_field[RTF_MINUTES].value;
         etm.et_tm.tm_sec = this->rt_field[RTF_SECONDS].value;
 
-        auto epoch_secs = std::chrono::seconds(tm2sec(&etm.et_tm));
+        auto epoch_secs = std::chrono::seconds(etm.to_timeval().tv_sec);
         retval
             = std::chrono::duration_cast<std::chrono::microseconds>(epoch_secs)
                   .count();
