@@ -2,6 +2,7 @@
 
 export YES_COLOR=1
 
+export TZ=UTC
 export HOME="./meta-sessions"
 export XDG_CONFIG_HOME="./meta-sessions/.config"
 rm -rf "./meta-sessions"
@@ -28,6 +29,18 @@ if test -d meta-sessions/.lnav; then
     exit 1
 fi
 
+# tag was saved and search finds comment
+run_cap_test ${lnav_test} -n \
+    -c ":load-session" \
+    -c "/Hello, World" \
+    ${test_dir}/logfile_access_log.0
+
+# tag was saved and search finds tag
+run_cap_test ${lnav_test} -n \
+    -c ":load-session" \
+    -c "/foo" \
+    ${test_dir}/logfile_access_log.0
+
 # tag was saved and :write-to displays the comments/tags
 run_cap_test ${lnav_test} -n \
     -c ":load-session" \
@@ -108,3 +121,28 @@ run_cap_test ${lnav_test} -n -f- \
 
 This is `markdown` now!
 EOF
+
+run_cap_test ${lnav_test} -n \
+    -c ":goto 46" \
+    -c ":tag bro-test" \
+    -c ":save-session" \
+    -c ";SELECT log_line, log_tags FROM bro_http_log WHERE log_tags IS NOT NULL" \
+    ${test_dir}/logfile_bro_http.log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ":load-session" \
+    -c ";SELECT log_line, log_tags FROM bro_http_log WHERE log_tags IS NOT NULL" \
+    ${test_dir}/logfile_bro_http.log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";UPDATE access_log SET log_annotations = '1' WHERE log_line = 0" \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";UPDATE access_log SET log_annotations = '{\"abc\": \"def\"' WHERE log_line = 0" \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";UPDATE access_log SET log_annotations = '{\"abc\": \"def\"}' WHERE log_line = 0" \
+    -c ";SELECT log_line,log_annotations FROM access_log WHERE log_annotations IS NOT NULL" \
+    ${test_dir}/logfile_access_log.0
