@@ -1,5 +1,6 @@
 #! /bin/bash
 
+export TZ=UTC
 export YES_COLOR=1
 unset XDG_CONFIG_HOME
 
@@ -18,6 +19,14 @@ run_cap_test ${lnav_test} -n \
     -c ";UPDATE lnav_view_files SET visible=0 WHERE endswith(filepath, 'log.0')" \
     ${test_dir}/logfile_access_log.*
 
+run_cap_test ${lnav_test} -n \
+    -c ";INSERT INTO lnav_view_files VALUES ('log', '/abc/def', 1)" \
+    ${test_dir}/logfile_access_log.*
+
+run_cap_test ${lnav_test} -n \
+    -c ";DELETE FROM lnav_view_files" \
+    ${test_dir}/logfile_access_log.*
+
 run_test ${lnav_test} -n \
     -c ";DELETE FROM lnav_view_stack" \
     ${test_dir}/logfile_access_log.0
@@ -34,6 +43,9 @@ run_cap_test ${lnav_test} -n \
     -c ";DELETE FROM lnav_view_stack WHERE name = 'log'" \
     ${test_dir}/logfile_access_log.0
 
+run_cap_test ${lnav_test} -nN \
+    -c ";INSERT INTO lnav_view_filters VALUES ('log', 0, 1, 'out', 'bad', '')"
+
 run_cap_test ${lnav_test} -n \
     -c ";INSERT INTO lnav_view_filters VALUES ('log', 0, 1, 'out', 'regex', '')" \
     ${test_dir}/logfile_access_log.0
@@ -94,6 +106,11 @@ run_cap_test ${lnav_test} -n \
     -c ";UPDATE lnav_view_filters SET pattern = 'vmkboot'" \
     ${test_dir}/logfile_access_log.0
 
+run_cap_test ${lnav_test} -n \
+    -c ":filter-out vmk" \
+    -c ";UPDATE lnav_view_filters SET enabled = 0" \
+    ${test_dir}/logfile_access_log.0
+
 run_test ${lnav_test} -n \
     -c ":filter-out vmk" \
     -c ";SELECT * FROM lnav_view_filter_stats" \
@@ -121,7 +138,7 @@ run_test ${lnav_test} -n \
 
 check_output "delete from lnav_views table works?" <<EOF
 count(*)
-8
+9
 EOF
 
 
@@ -133,7 +150,7 @@ run_test ${lnav_test} -n \
 
 check_output "insert into lnav_views table works?" <<EOF
 count(*)
-8
+9
 EOF
 
 run_cap_test ${lnav_test} -n \
@@ -177,3 +194,34 @@ run_cap_test ${lnav_test} -n \
     -c ";SELECT top_meta FROM lnav_top_view" \
     -c ":write-json-to -" \
     ${test_dir}/logfile_xml_msg.0
+
+run_cap_test ${lnav_test} -n -I ${test_dir} \
+    -c ";UPDATE lnav_views SET options = json_object('row-details', 'show') WHERE name = 'log'" \
+    -c ":goto 2" \
+    ${test_dir}/logfile_xml_msg.0
+
+run_cap_test ${lnav_test} -n -I ${test_dir} \
+    -c ";UPDATE lnav_views SET options = json_object('row-details', 'show') WHERE name = 'log'" \
+    -c ":goto 9" \
+    ${test_dir}/logfile_bunyan.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";UPDATE lnav_views SET options = json_object('row-time-offset', 'show') WHERE name = 'log'" \
+    ${test_dir}/logfile_w3c_big.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";UPDATE lnav_views SET top_meta = json_object('file', 'bad') WHERE name = 'text'" \
+    ${test_dir}/textfile_ansi.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";SELECT * FROM lnav_views" \
+    -c ":write-json-to -" \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";INSERT INTO lnav_view_filters (view_name, language, pattern) VALUES ('text', 'sql', ':sc_bytes = 134')" \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ";INSERT INTO lnav_view_filters (view_name, language, pattern) VALUES ('log', 'sql', ':sc_bytes # 134')" \
+    ${test_dir}/logfile_access_log.0
